/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "desktopwidget.h"
#include "topbar.h"

#include <KDE/KWindowSystem>
#include <KDE/KDirWatch>
#include <KDE/KStandardDirs>
#include <KDE/KUrlPixmapProvider>
#include <KDE/KRun>
#include <KDE/KIconLoader>
#include <KDE/KDesktopFile>
#include <KDE/KMenu>
#include <KDE/KStandardDirs>
#include <KDE/KGlobal>

//#include <konq_popupmenu.h>

#include <QPainter>
#include <QApplication>
#include <QDesktopWidget>
#include <QPalette>
#include <QtDBus>
#include <QDir>
#include <QFileInfo>
#include <QDesktopServices>
#include <QMouseEvent>
#include <QFile>
#include <QApplication>

#include <kdebug.h>

DesktopIconWidget::DesktopIconWidget( const QString &path, QWidget *parent ) : QWidget( parent ), m_path(path)
{
    setFixedSize(100, 50);
    updateIcon();
}

void DesktopIconWidget::updateIcon()
{
    if( KDesktopFile::isDesktopFile( m_path ) )
    {
        KDesktopFile ds(m_path);
        m_image = DesktopIcon( ds.readIcon(), 32 );
        m_name = ds.readName();
    }
    else
    {
        QFileInfo fileInfo(m_path);
        KUrlPixmapProvider imgLoader;
        m_image = imgLoader.pixmapFor( m_path, 32 );
        m_name = fileInfo.fileName();
    }
    update();
}

void DesktopIconWidget::mousePressEvent(QMouseEvent * ev)
{
    if (ev->button() == Qt::RightButton)
    {
       // KonqPopupMenu menu();
       // menu.exec();
    }

    if (ev->button() == Qt::LeftButton)
    {
        m_mousepos = ev->pos();
        grabMouse(QCursor(Qt::SizeAllCursor));
    }
}

void DesktopIconWidget::mouseMoveEvent(QMouseEvent * ev)
{
    QPoint p = ev->globalPos() - m_mousepos;
    move(p.x(), p.y());
}

void DesktopIconWidget::mouseReleaseEvent(QMouseEvent * ev)
{
    releaseMouse();
    // update location in the config file...
}

void DesktopIconWidget::mouseDoubleClickEvent(QMouseEvent * ev)
{
    new KRun( KUrl(m_path), this);
}

void DesktopIconWidget::paintEvent(QPaintEvent * ev)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setWindow(-50, -50, 100, 50);
    painter.drawPixmap(QRect(-16, -50, 32, 32), m_image, QRect(0, 0, m_image.width(), m_image.height()));
    painter.drawText(-50, -15, 100, 20, Qt::AlignHCenter, m_name );
}

DesktopWidget::DesktopWidget( QWidget *parent ) : QWidget(parent), m_size(26)
{
    setObjectName("Desktop");
    connect( QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(slotDesktopResized(int)));
    KWindowSystem::setType( winId(), NET::Desktop );
    KWindowSystem::setOnAllDesktops( winId(), true );

    m_topBar = new TopBar(0);
    m_topBar->show();

    m_bottomBar = new BottomBar(0);
    m_bottomBar->show();

    slotDesktopResized( QApplication::desktop()->primaryScreen() );

    m_desktopWatcher = new KDirWatch(this);
    connect( m_desktopWatcher, SIGNAL(created(QString)), this, SLOT(slotFileCreated(QString)));
    connect( m_desktopWatcher, SIGNAL(deleted(QString)), this, SLOT(slotFileDeleted(QString)));
    connect( m_desktopWatcher, SIGNAL(dirty(QString)), this, SLOT(slotFileChanged(QString)));

    m_styleWatcher = new KDirWatch(this);
    if( !KGlobal::dirs()->locate("data","blazer/styles.css").isEmpty() )
        m_styleWatcher->addFile(KGlobal::dirs()->locate("data","blazer/styles.css"));
    connect( m_styleWatcher, SIGNAL(created(QString)), this, SLOT(slotUpdateStyleSheet()));
    connect( m_styleWatcher, SIGNAL(deleted(QString)), this, SLOT(slotUpdateStyleSheet()));
    connect( m_styleWatcher, SIGNAL(dirty(QString)), this, SLOT(slotUpdateStyleSheet()));

    KDirWatch::self()->addDir(QDesktopServices::storageLocation(QDesktopServices::DesktopLocation), KDirWatch::WatchFiles );
    populateDesktop(QDesktopServices::storageLocation(QDesktopServices::DesktopLocation));
    QDBusConnection::sessionBus().registerObject("/Desktop", this);

    m_lastPos = QPoint(100,50);
    m_lastPos.setY( m_lastPos.y() + m_size);

}

DesktopWidget::~DesktopWidget()
{
    delete m_topBar;
    delete m_bottomBar;
}

void DesktopWidget::slotDesktopResized ( int screen )
{
    setGeometry(0, 0, QApplication::desktop()->width(), QApplication::desktop()->height() );
    m_topBar->slotDesktopResized( screen, m_size );
    m_bottomBar->slotDesktopResized( screen, m_size );
}

void DesktopWidget::slotFileCreated( const QString &path )
{
    m_lastPos.setY( m_lastPos.y() + 105 );
    if( m_lastPos.y() > height() - ( m_size + 100 ) )
    {
        m_lastPos.setY(m_size);
        m_lastPos.setX(m_lastPos.x() + 55 );
    }

    QFileInfo fileInfo(path);
    if( m_icons.contains(fileInfo.absoluteFilePath()) )
        delete m_icons[fileInfo.absoluteFilePath()];
    m_icons[fileInfo.absoluteFilePath()] = new DesktopIconWidget( fileInfo.absoluteFilePath(), this );
    m_icons[fileInfo.absoluteFilePath()]->move( m_lastPos );
    m_icons[fileInfo.absoluteFilePath()]->show();
}

void DesktopWidget::slotFileDeleted( const QString &path )
{
    QFileInfo fileInfo(path);
    if( m_icons.contains(fileInfo.absoluteFilePath()) )
        delete m_icons[fileInfo.absoluteFilePath()];
    m_icons.remove(fileInfo.absoluteFilePath());
}

void DesktopWidget::slotFileChanged( const QString &path )
{
    if( m_icons.contains(path) )
            m_icons[path]->updateIcon();
}

void DesktopWidget::populateDesktop( const QString &path )
{
    QDir desktopDir( path );
    foreach( QFileInfo fileInfo, desktopDir.entryInfoList( QDir::AllEntries | QDir::NoDotAndDotDot ) )
    {
        m_lastPos.setY( m_lastPos.y() + 105 );
        if( m_lastPos.y() > height() - ( m_size + 100 ) )
        {
            m_lastPos.setY(m_size);
            m_lastPos.setX(m_lastPos.x() + 55 );
        }

        if( m_icons.contains(fileInfo.absoluteFilePath()) )
            delete m_icons[fileInfo.absoluteFilePath()];
        m_icons[fileInfo.absoluteFilePath()] = new DesktopIconWidget( fileInfo.absoluteFilePath(), this );
        m_icons[fileInfo.absoluteFilePath()]->move( m_lastPos );
        m_icons[fileInfo.absoluteFilePath()]->show();
    }

}

void  DesktopWidget::slotUpdateStyleSheet( )
{
    QString styleSheet = KGlobal::dirs()->locate("data","blazer/styles.css");
    kDebug() << "update stylesheet" << styleSheet;

    QFile styleSheetFile(styleSheet);
    if( styleSheetFile.open(QIODevice::ReadOnly) )
    {
      QString styleSheetContent = styleSheetFile.readAll();
      qApp->setStyleSheet( styleSheetContent );
    }
}
