/************************************************************************
*  This file is part of the KDE project, module kcontrol.               *
*  Copyright (C) 2007 Ian Ventura-Whiting <fizz@titania.co.uk>          *
*                                                                       *
*  You can Freely distribute this program under the GNU LGPL version 2  *
*************************************************************************/

#ifndef __DSN_Dialog_h_Included__
#define __DSN_Dialog_h_Included__

#include <QObject>
#include <KPageDialog>
#include <odbcinstext.h>

class QString;
class QTreeWidget;
class QRadioButton;
class KPageWidgetItem;
class KPushButton;
class QWidget;


/**
 * @short ODBC Data Source Configuration Dialog.
 *
 * This class provides an ODBC data source configuration dialog and
 * is used by the Data Sources KCM. The class provides add, modify
 * and view of user, system or file data sources.
 * The class is configured using a number of set methods. Once ready
 * the setup method must be called.
 *
 * @author Ian Ventura-Whiting fizz @ titania.co.uk
 */


// DSN Dialog Class
class DSNDialog: public KPageDialog
{
	Q_OBJECT

	public:
		/**
		 * Creates a new data source dialog.
		 */
		DSNDialog(QWidget *parent);

		/**
		 * Destroy the data source dialog.
		 */
		~DSNDialog();

		/**
		 * Setup the data source dialog. Use this after calling the
		 * various class set methods, such as setDialogMode and
		 * setODBCType.
		 *
		 * @returns The return value is true is successful, false
		 *          indicates that there was a problem, such as no
		 *          ODBC drivers have been configured.
		 */
		bool setup();

		/**
		 *  @li @p Add     - A dialog to add a new data source.
		 *  @li @p Modify  - A dialog to modify an existing data source.
		 *  @li @p View    - A dialog to view an existing data source.
		 */
		enum DSNMode
		{
			Add,
			Modify,
			View
		};

		/**
		 * Sets the mode of the data source dialog.
		 */
		void setDialogMode(DSNMode mode);

		/**
		 *  @li @p User    - The DSN is a user data source.
		 *  @li @p System  - The DSN is a system data source.
		 *  @li @p File    - The DSN is a file data source.
		 */
		enum DSNType
		{
			User,
			System,
			File
		};

		/**
		 * Sets the type of data source. User, System or File.
		 */
		void setODBCType(DSNType type);

		/**
		 * Sets the data source name. This is only needed when
		 * modifying or viewing an existing data source.
		 */
		void setDSNName(QString name);

		/**
		 * Sets the file data source directory name. This is only
		 * needed when working with file data sources.
		 */
		void setDSNDir(QString dirName);

	private slots:
		void dsnSelectionChanged(bool state);
		void nextClicked();
		void previousClicked();

	private:
		// Functions...
		void getProperties();
		bool saveProperties();
		void buildInterface();

		// Widgets...
		QRadioButton *userDSNRadio;
		QRadioButton *systemDSNRadio;
		QRadioButton *fileDSNRadio;
		QTreeWidget *driverList;
		QWidget *finishWidget;
		KPushButton *testDSNButton;

		// Wizard Pages...
		KPageWidgetItem *dsnPage;
		KPageWidgetItem *driverPage;
		KPageWidgetItem *basicPage;
		KPageWidgetItem *advancedPage;
		KPageWidgetItem *loggingPage;
		KPageWidgetItem *finishPage;

		// Properties...
		DSNMode dialogMode;
		DSNType odbcType;
		QString dsnName;
		QString dsnDir;

		// Internal...
		HODBCINSTPROPERTY odbcProperties;	// ODBC properties
		HODBCINSTPROPERTY odbcPointer;		// ODBC properties pointer
};

#endif // __DSN_Dialog_h_Included__
