/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef KAUTHORIZE_H
#define KAUTHORIZE_H

#include <QtCore/QObject>
#include <QtCore/QProcess>
#include <QtCore/QCoreApplication>

class KAuthorize : public QObject
{
    Q_OBJECT

public:
    enum Result {
        Authorized = 0x01,
        NotAuthorized = 0x02,
        RequiresAuthentication = 0x03,
        Error = 0x04
    };

    KAuthorize(const QString &actionId);
    virtual ~KAuthorize();

    /**
     * \brief Obtains authorization for the current action.
     *
     * This method tries to get an authorization for the application specified in the
     * pid parameter. You usually want this to be the current application's PID.
     *
     * This function will also take care of calling the authorization dialog
     * and let the user authenticate, if needed. It will return immediately, and the
     * result will be streamed through the resultObtained signal
     *
     * \param pid Process id of the application in question
     */
    void obtainAuthorization(Q_PID pid = QCoreApplication::applicationPid());

    /**
     * This function should be used by mechanisms (e.g.: helper applications).
     * It returns the action should be carried out, so if the caller was
     * actually authorized to perform it. The result is in form of a Result, so that
     * you can have more control over the whole process, and detect an eventual error.
     * Most of the times you simply want to check if the result is == to Result::Authorized,
     * if you don't have specific needs.
     *
     * It is CRITICAL that you call this function
     * and check what it returns before doing anything in your helper, since otherwise
     * you could be actually performing an action from an unknown or unauthorized caller.
     *
     * The revokeIfOneShot parameter is very important: if it is set to true, if the authorization
     * was a One Shot authorization, it will be revoked right after calling this function.
     * To the bottom line, you should set this parameter to true if you're the mechanism
     * (so you're actually carrying out the action), to false if you're not (for example,
     * if you only want to check if the caller is authorized but not perform the action).
     *
     * \note The \c pid parameter refers to the application we want to check authorization for.
     *
     * \param pid the pid of the caller we want to check authorization for
     * \param revokeIfOneShot \c true  if we're carrying out the action, so we want the auth
     *                                 to be revoked right after
     *                        \c false if we're not carrying out the action, so we don't want
     *                                 the auth to be revoked right after
     *
     */
    void isApplicationAuthorized(Q_PID pid, bool revokeIfOneShot);

    /**
     * \brief Obtains authorization for an action.
     *
     * The same as the non-static one, except for the fact that this method is
     * synchronous. In most cases, you would want to call this method instead
     * of constructing your own KAuthorize object.
     *
     * \param actionId the id of the action to request authorization for
     * \param winId the X window id for the request (use 0 if there's no window)
     * \param pid Process id of the application in question
     */
    static Result obtainAuthorization(const QString &actionId, Q_PID pid = QCoreApplication::applicationPid());

    /**
     * The same as the non-static one, except for the fact that this method is
     * synchronous. In most cases, you would want to call this method instead
     * of constructing your own KAuthorize object.
     *
     * \param actionId the id of the action to request authorization for
     * \param pid the pid of the caller we want to check authorization for
     * \param revokeIfOneShot \c true  if we're carrying out the action, so we want the auth
     *                                 to be revoked right after
     *                        \c false if we're not carrying out the action, so we don't want
     *                                 the auth to be revoked right after
     *
     * \return \c Result::Authorized if the caller is authorized and the action should be performed
     *         \c otherwise if the caller was not authorized and the action should not be performed,
     *                      or an error has occurred
     *
     */
    static Result isApplicationAuthorized(const QString &actionId, Q_PID pid, bool revokeIfOneShot);

Q_SIGNALS:
    /**
     * This signal streams the result of a request
     */
    void resultObtained(Result result);

};

#endif // KAUTHORIZE_H
