/*
    This file is part of Fate.

    Copyright (c) 2005 SUSE LINUX Products GmbH
    
    Author: Cornelius Schumacher <cschum@suse.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "kupdate.h"

#include "kinstalleriface_stub.h"

#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kmainwindow.h>
#include <kaction.h>
#include <kstatusbar.h>
#include <kapplication.h>
#include <dcopclient.h>
#include <kprocess.h>
#include <kstandarddirs.h>

#include <qpushbutton.h>
#include <qlayout.h>
#include <qtextview.h>
#include <qlabel.h>
#include <qfile.h>
#include <qtimer.h>

#include <unistd.h>

KUpdateDialog::KUpdateDialog( QWidget *parent, KUpdate *u )
  : KDialogBase( parent, 0, true, i18n("Update Application"), Ok, Ok, true ),
    mUpdate( u )
{
  QFrame *topFrame = makeMainWidget();

  QBoxLayout *topLayout = new QVBoxLayout( topFrame );
  topLayout->setSpacing( spacingHint() );

  mView = new QTextView( topFrame );
  topLayout->addWidget( mView );
  
  QHBoxLayout *buttonLayout = new QHBoxLayout( topLayout );
    
  buttonLayout->addStretch( 1 );
    
  mUpdateButton = new QPushButton( i18n("Update"), topFrame );
  buttonLayout->addWidget( mUpdateButton );
  connect( mUpdateButton, SIGNAL( clicked() ), SLOT( update() ) );

  mUpdate->checkForUpdate();

  QString txt;
  if ( mUpdate->errorMessage().isEmpty() ) {
    txt = "<h1>" + mUpdate->appname() + "</h1>";
    txt += "<p><b>Currently installed version:</b> " + mUpdate->currentVersion()
      + "</p>";
    txt += "<p><b>Newest version:</b> " + mUpdate->newestVersion()
      + "</p>";

    mUpdateButton->setEnabled( mUpdate->updateAvailable() );
  } else {
    txt = "<em><b>Error:</b> " + mUpdate->errorMessage() + "</em>";
  
    mUpdateButton->setEnabled( false );
  }

  mView->setText( txt );

  resize( 400, 300 );
}

void KUpdateDialog::update()
{
  mUpdate->update();
}


StatusBarLabel::StatusBarLabel()
  : QLabel( 0 )
{
  mBgColor = paletteBackgroundColor();
  mFgColor = paletteForegroundColor();

  setState( Undefined );
}

void StatusBarLabel::mousePressEvent( QMouseEvent * )
{
  emit clicked();
}

void StatusBarLabel::setState( State state )
{
  switch( state ) {
    case Uptodate:
      setText( i18n("Up-to-date") );
      break;
    case UpdateAvailable:
      setText( i18n("Update available") );
      break;
    case NoKinstaller:
      setText( i18n("No KInstaller") );
      break;
    case Connecting:
      setText( i18n("Connecting...") );
      break;
    case Undefined:
      setText( QString::null );
      break;
  }
      
  if ( state == UpdateAvailable ) {
    setPaletteBackgroundColor( "#aa0000" );
    setPaletteForegroundColor( "#ffffff" );
  } else {
    setPaletteBackgroundColor( mBgColor );
    setPaletteForegroundColor( mFgColor );
  }
}


AppStarter::AppStarter( const QCString &appname )
  : mAppname( appname )
{
  kdDebug() << "AppStarter() " << appname << endl;

  DCOPClient *c = kapp->dcopClient();
  if ( !c->isApplicationRegistered( mAppname ) ) {
    kdDebug() << "START" << endl;
    c->setNotifications( true );
    connect( c, SIGNAL( applicationRegistered( const QCString & ) ),
      SLOT( slotApplicationRegistered( const QCString & ) ) );
    KProcess *proc = new KProcess;
    *proc << appname;
    connect( proc, SIGNAL( processExited( KProcess * ) ),
      SLOT( destroyProcess( KProcess * ) ) );
    proc->start();
  } else {
    QTimer::singleShot( 0, this, SLOT( emitAppStarted() ) );
  }
}

void AppStarter::emitAppStarted()
{
  kdDebug() << "AppStarter::emitAppStarted()" << endl;

  emit appStarted( this );
}

void AppStarter::slotApplicationRegistered( const QCString & )
{
  DCOPClient *c = kapp->dcopClient();  
  c->setNotifications( false );
  disconnect( c,
    SIGNAL( applicationRegistered( const QCString & ) ),
    this, SLOT( slotApplicationRegistered( const QCString & ) ) );

  emitAppStarted();
}

void AppStarter::destroyProcess( KProcess *p )
{
  delete p;
}


KUpdate::KUpdate( KMainWindow *mainWindow, const QString &appname )
  : DCOPObject( "KUpdate" ), QObject( mainWindow ), mMainWindow( mainWindow ),
    mAppname( appname )
{
}

void KUpdate::initAction( KMainWindow *m, KActionCollection *actionCollection )
{
  kdDebug() << "KUpdate::initAction()" << endl;
  new KAction( i18n("Update..."), 0, m, SLOT( showDialog() ),
    actionCollection, "update" );
}

void KUpdate::init()
{
  bool enabled = true;

  if( enabled ) {
    new KAction( i18n("Update..."), 0, this, SLOT( showDialog() ),
      mMainWindow->actionCollection(), "update" );
  }

  mStatusLabel = new StatusBarLabel();
  connect( mStatusLabel, SIGNAL( clicked() ), SLOT( showDialog() ) );

  mMainWindow->statusBar()->addWidget( mStatusLabel, 0, true );

  if ( enabled ) mStatusLabel->show();
  else mStatusLabel->hide();

  mStatusLabel->setState( StatusBarLabel::Connecting );

  mStatusLabel->hide();

  AppStarter *a = new AppStarter( "kinstaller" );
  connect( a, SIGNAL( appStarted( AppStarter * ) ),
    SLOT( checkForUpdate() ) );
  connect( a, SIGNAL( appStarted( AppStarter * ) ),
    SLOT( destroyAppStarter( AppStarter * ) ) );
}

void KUpdate::destroyAppStarter( AppStarter *a )
{
  delete a;
}

void KUpdate::checkForUpdate()
{
  kdDebug() << "KUpdate::checkForUpdate()" << endl;

  KInstallerIface_stub kinstaller( "kinstaller", "AppManager" );
  mCurrentVersion = kinstaller.installedVersion( mAppname );
  if ( !kinstaller.ok() ) {
    mStatusLabel->setState( StatusBarLabel::NoKinstaller );
    mErrorMessage = i18n("KInstaller is not running.");
    kdDebug() << "DCOP Call to KInstaller::installedVersion() failed" << endl;
    return;
  }
  mNewestVersion = kinstaller.newestVersion( mAppname );
  if ( !kinstaller.ok() ) {
    mStatusLabel->setState( StatusBarLabel::NoKinstaller );
    mErrorMessage = i18n("KInstaller is not running.");
    kdDebug() << "DCOP Call to KInstaller::newestVersion() failed" << endl;
    return;
  }

  mUpdateAvailable = kinstaller.hasNewerVersion( mAppname, mCurrentVersion );

  mStatusLabel->show();

  if ( mUpdateAvailable ) {
    mStatusLabel->setState( StatusBarLabel::UpdateAvailable );
  } else {
    mStatusLabel->setState( StatusBarLabel::Uptodate );
  }

  mErrorMessage = QString::null;
}

QString KUpdate::errorMessage()
{
  return mErrorMessage;
}

QString KUpdate::currentVersion()
{
  return mCurrentVersion;
}

QString KUpdate::newestVersion()
{
  return mNewestVersion;
}

QString KUpdate::appname()
{
  return mAppname;
}

bool KUpdate::updateAvailable()
{
  return mUpdateAvailable;
}

void KUpdate::update()
{
  kdDebug() << "KUpdate::update()" << endl;

  KInstallerIface_stub kinstall( "kinstaller", "AppManager" );
  kinstall.update( mAppname );
  if ( !kinstall.ok() ) {
    KMessageBox::sorry( 0, i18n("DCOP Call to KInstaller::update() failed") );
    return;
  }
}

void KUpdate::showDialog()
{
  KUpdateDialog dlg( 0, this );
  dlg.exec();
}

void KUpdate::restart()
{
  kdDebug() << "KUpdate::restart()" << endl;

  KMessageBox::information( 0, i18n("Now restarting %1.").arg( mAppname ) ); 

  // do this on a timer to give us time to return true
  QTimer::singleShot(0, this, SLOT(slotRestart()));
}

void KUpdate::slotRestart()
{
  char ** o_argv = new char *[ 2 ];
  o_argv[ 0 ] = strdup( mAppname.utf8() );
  o_argv[ 1 ] = 0;
  execv( QFile::encodeName( locate( "exe", "kdeinit4_wrapper") ), o_argv );

  exit( 1 );
}

#include "kupdate.moc"
