/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "historyfile.h"

#include <QFile>

#include <KDebug>
#include <KConfig>
#include <KConfigGroup>

class HistoryFilePrivate
{
public:
    HistoryFilePrivate()
        : config(0), isConfigWritable(false) {}

    QString fileName;
    ClipboardEntryList entryList;
    KConfig* config;
    bool isConfigWritable;
};

HistoryFile::HistoryFile(const QString& fileName)
    : d(new HistoryFilePrivate)
{
    if (!fileName.isEmpty())
        setFile(fileName, CreateIfAbsentMode);
    else
        kDebug() << "warning: emtpy filename passed to constructor, this won't work";
}

HistoryFile::~HistoryFile()
{
    close();
    delete d;
}

bool HistoryFile::setFile(const QString& fileName, FileOpenMode openMode)
{
    close();

    d->fileName = fileName;

    // cleanup. if some file loading error occurs we'll be clean
    d->entryList.clear();

    if (d->fileName.isEmpty())
    {
        kDebug() << "Warning: refusing to work with file which has an empty name";
        return false;
    }

    if (openMode == OpenExistingMode && !QFile::exists(d->fileName))
    {
        kDebug() << "existing file not found while tried to open in OpenExisting mode";
        return false;
    }

    kDebug() << "opening historyfile:" << fileName;
    d->config = new KConfig(fileName, KConfig::SimpleConfig);

    if (!d->config)
    {
        kDebug() << "failed to open file";
        return false;
    }

    d->isConfigWritable = d->config->isConfigWritable(false);

    if (!d->isConfigWritable)
    {
        kDebug() << "can't write to file" << fileName;
        return false;
    }

    // read entries
    QStringList groups = d->config->groupList();
    // reserve space in list
    int count = groups.count();
    for (int i=0; i<count;++i)
        d->entryList.append(ClipboardEntry());

    foreach (const QString& groupName, groups)
    {
        ClipboardEntry entry;
        KConfigGroup entryGroup(d->config, groupName);
        entry.uuid = groupName;
        entry.isText = entryGroup.readEntry("IsText", true);
        entry.isSelection = entryGroup.readEntry("IsSelection", true);
        if (entry.isText)
            entry.data = entryGroup.readEntry("Data"); // read as string
        else
        {
            kDebug() << "READING OF IMAGE DATA IS NOT SUPPORTED YET";
            continue;
        }

        int index = entryGroup.readEntry("Index", -1);
        if (index < 0 || index >= count)
        {
            kDebug() << "entry index out of range, skipping it";
            continue;
        }

        d->entryList[index] = entry;
    }

    return true;
}

int HistoryFile::entryCount() const
{
    return d->entryList.count();
}

bool HistoryFile::addEntry(const ClipboardEntry& entry)
{
    if (d->fileName.isEmpty())
    {
        kDebug() << "history file has no filename assigned";
        return false;
    }

    if (!d->isConfigWritable)
    {
        kDebug() << "can not add entry - file is unwritable";
        return false;
    }

    if (entry.uuid.isEmpty())
    {
        kDebug() << "refusing to add entry with empty uuid";
        return false;
    }

    if (!entry.isText)
    {
        kDebug() << "WRITING OF IMAGE DATA IS NOT SUPPORTED YET";
        return false;
    }

    d->entryList.append(entry);
    KConfigGroup entryGroup(d->config, entry.uuid);

    entryGroup.writeEntry("Data", entry.data);
    entryGroup.writeEntry("IsText", entry.isText);
    entryGroup.writeEntry("IsSelection", entry.isSelection);
    entryGroup.writeEntry("Index", d->entryList.count()-1);

    // NOTE: Think if this really should be done? This way we're crash proof, but how efficient is
    // this? If this is gonna change, need to adjust tests
    d->config->sync();

    return true;
}

bool HistoryFile::addEntries(const ClipboardEntryList& entryList)
{
    bool res = false;
    foreach (const ClipboardEntry& entry, entryList)
    {
        res = addEntry(entry);
        if (!res)
            break;
    }

    return res;
}

bool HistoryFile::removeEntry(int index)
{
    if (index < 0 || index >= d->entryList.count())
    {
        kDebug() << "error: wrong entry index passed";
        return false;
    }
    if (!d->config)
    {
        kDebug() << "config object is null";
        return false;
    }

    d->config->deleteGroup(d->entryList.at(index).uuid);
    d->entryList.removeAt(index);

    return true;
}

bool HistoryFile::removeEntry(const QString& uuid)
{
    int count = entryCount();
    for (int i=0; i<count; ++i)
    {
        if (d->entryList[i].uuid == uuid)
            return removeEntry(i);
    }

    return false;
}

void HistoryFile::clear()
{
    if (!d->config)
    {
        kDebug() << "config object is null";
        return;
    }

    int count = d->entryList.count();
    for (int i=0; i<count; ++i)
        d->config->deleteGroup(d->entryList.at(i).uuid);

    d->entryList.clear();
}

void HistoryFile::close()
{
    if (d->config)
        d->config->sync();

    delete d->config;
    d->config = 0;

    d->entryList.clear();
    d->fileName = QString();
    d->isConfigWritable = false;
}

ClipboardEntry HistoryFile::entry(int index) const
{
    return d->entryList.at(index);
}

ClipboardEntryList HistoryFile::entries() const
{
    return d->entryList;
}

bool HistoryFile::isWritable() const
{
    return !d->fileName.isEmpty() && d->isConfigWritable;
}
