/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef HISTORY_FILE_H
#define HISTORY_FILE_H

#include <QString>

#include "klipper_export.h"
#include "clipboardtypes.h"

class HistoryFilePrivate;

/**
 * This class represents a klipper history file.
 * It handles opening, loading and saving history while abstracting
 * an actual file format
 */
class KLIPPER_EXPORT HistoryFile
{
public:
    enum FileOpenMode
    {
        CreateIfAbsentMode,
        OpenExistingMode
    };

    /**
     * Constructor. If @p fileName is passed, file loaded if it exists or will be created after
     * content changes
     */
    explicit HistoryFile(const QString& fileName = QString());

    ~HistoryFile();

    /**
     * Sets a file to operate with.
     * If file does not exist it will be created after any entry gets added
     *
     * @return true if file was successfully created/loaded
     */
    bool setFile(const QString& fileName, FileOpenMode openMode);

    /**
     * @return number of entries in history
     */
    int entryCount() const;

    /**
     * Adds a new entry to history
     *
     * @return whether entry was successfully added
     */
    bool addEntry(const ClipboardEntry& entry);

    /**
     * Adds an entry list to history
     *
     * @return whether entries were successfully added
     */
    bool addEntries(const ClipboardEntryList& entryList);

    /**
     * Removes an entry from history
     * @param index index of the entry to remove
     *
     * @return whether entry was successfully removed
     */
    bool removeEntry(int index);

    /**
     * Removes an entry from history
     * @param uuid uuid of the entry to remove
     *
     * @return whether entry was successfully removed
     */
    bool removeEntry(const QString& uuid);

    /**
     * Removes all entries from file
     */
    void clear();

    /**
     * Closes file and resets this object.
     * All operations will fail until new file is opened using setFile
     */
    void close();

    /**
     * @return an entry from history file at certain index
     */
    ClipboardEntry entry(int index) const;

    /**
     * @return all entries contained in file
     */
    ClipboardEntryList entries() const;

    /**
     * @returns true if this file is writable and in an operatable state (not closed, writes
     * permitted)
     */
    bool isWritable() const;

private:
    HistoryFilePrivate * const d;
};

#endif
