/*
 *  Copyright (C) 2008 by Tom Albers <tomalbers@kde.nl>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 **/

// local
#include "plugin_bookmarksync.h"
#include "ui_settings.h"

// Qt
#include <QTimer>

// KDE
#include <KActionCollection>
#include <KActionMenu>
#include <KBookmarkManager>
#include <KDirWatch>
#include <KFileDialog>
#include <KMessageBox>
#include <KStandardDirs>
#include <KGenericFactory>

// KIO
#include <kio/job.h>
#include <kio/udsentry.h>
#include <kio/netaccess.h>

K_PLUGIN_FACTORY( PluginBookmarkSyncFactory, registerPlugin<PluginBookmarkSync>();)
K_EXPORT_PLUGIN( PluginBookmarkSyncFactory( "bookmarksync" ) )

class SettingsDialog : public KDialog, public Ui::Settings
{
public:
    SettingsDialog( QWidget *parent ) : KDialog( parent ) {
        setupUi( mainWidget() );
        setWindowTitle( i18n( "Settings" ) );
        setButtons( Ok | Cancel );
        connect( this, SIGNAL( cancelClicked() ), this, SLOT( reject() ) );
    }
};

PluginBookmarkSync::PluginBookmarkSync( QObject* parent, const QVariantList & )
        : KParts::Plugin( parent ), m_localSize( -1 ), m_autoFetchInProgress(false)
{
    // parent
    m_parent = 0;   // object cast to mainwindow or somthing like that?

    // get the size, needs m_parent.
    m_localSize = getLocalSize();

    // read the settings
    m_config = new KConfig( "bookmarksync" );
    KConfigGroup group = m_config->group( "Settings" );
    m_location = group.readEntry( "Location",KUrl() );
    m_autoSave = group.readEntry( "AutoSave", true );
    m_autoFetch = group.readEntry( "AutoFetch", true );

    // Create the actions.
    m_menu = new KActionMenu( i18n( "Sync Bookmarks" ), actionCollection() );
    actionCollection()->addAction( "action bookmarksync", m_menu );
    m_menu->setDelayed( false );

    QAction *a = actionCollection()->addAction( "configure" );
    a->setText( i18n( "Configure..." ) );
    connect( a, SIGNAL( triggered() ), this, SLOT( slotConfigure() ) );
    m_menu->addAction( a );

    m_fetchAction = actionCollection()->addAction( "fetch" );
    m_fetchAction->setText( i18n( "Fetch bookmarks" ) );
    connect( m_fetchAction, SIGNAL( triggered() ), this, SLOT( slotFetch() ) );
    m_menu->addAction( m_fetchAction );
    if ( m_location.isEmpty() )
        m_fetchAction->setEnabled( false );

    m_storeAction = actionCollection()->addAction( "store" );
    m_storeAction->setText( i18n( "Save bookmarks" ) );
    connect( m_storeAction, SIGNAL( triggered() ), this, SLOT( slotSaveDoIt() ) );
    m_menu->addAction( m_storeAction );
    if ( m_location.isEmpty() )
        m_storeAction->setEnabled( false );

    // initialise the settings..
    m_settings = new SettingsDialog( m_parent );
    connect( m_settings->kpushbutton, SIGNAL( clicked( bool ) ), SLOT( slotGetLocation() ) );
    if ( !m_location.isEmpty() )
        m_settings->kpushbutton->setText( m_location.prettyUrl() );
    m_settings->AutoSave->setChecked( m_autoSave );
    m_settings->AutoFetch->setChecked( m_autoFetch );

    // Setup the Timers.
    m_saveTimer = new QTimer( m_parent );
    m_saveTimer->setSingleShot( true );
    connect( m_saveTimer, SIGNAL( timeout() ), SLOT( slotSaveDoIt() ) );

    // Check if the other version is newer
    if ( m_autoFetch ) {
        m_autoFetchInProgress = true;
        QTimer::singleShot( 10 * 1000, this, SLOT( slotAutoFetch() ) );
    }

    // Setup the watch
    m_watch = new KDirWatch( m_parent );
    m_watch->addFile( KStandardDirs::locateLocal( "data", "konqueror/bookmarks.xml" ) );
    connect( m_watch, SIGNAL( created( const QString& ) ), SLOT( slotSaveMaybe() ) );
    connect( m_watch, SIGNAL( dirty( const QString& ) ), SLOT( slotSaveMaybe() ) );
    m_autoSave && !m_autoFetchInProgress ? m_watch->startScan() : m_watch->stopScan();
}

PluginBookmarkSync::~PluginBookmarkSync()
{
    delete m_config;
}

void PluginBookmarkSync::slotAutoFetch()
{
    // Get timestamps;
    KIO::UDSEntry entryOther;
    KUrl bookmarksOther( m_location );
    bookmarksOther.addPath( "bookmarks.xml" );
    bool okOther = KIO::NetAccess::stat( bookmarksOther, entryOther, m_parent );

    KIO::UDSEntry entryHere;
    KUrl bookmarksHere( KStandardDirs::locateLocal( "data", "konqueror/bookmarks.xml" ) );
    bool okHere = KIO::NetAccess::stat( bookmarksHere, entryHere, m_parent );

    // If all ok, compare and take action.
    if ( okHere && okOther ) {
        time_t timeHere = entryHere.numberValue( KIO::UDSEntry::UDS_MODIFICATION_TIME, -1 );
        time_t timeOther = entryOther.numberValue( KIO::UDSEntry::UDS_MODIFICATION_TIME, -1 );
        int sizeHere = entryHere.numberValue( KIO::UDSEntry::UDS_SIZE, -1 );
        int sizeOther = entryOther.numberValue( KIO::UDSEntry::UDS_SIZE, -1 );
        kDebug() << "time: local: " << timeHere << "remote:" << timeOther;
        kDebug() << "size: local: " << sizeHere << "remote:" << sizeOther;
        if ( sizeHere == sizeOther ) {
            kDebug() << "File sizes are the same, nothing to do";
            m_autoFetchInProgress = false;
        } else if ( timeHere > timeOther ) {
            kDebug() << "Time local is newer than remote";
            if ( m_autoSave )
                slotSave();
        } else if ( timeOther > timeHere ) {
            kDebug() << "Time remote is newer than local, auto fetching";
            slotFetch();
        }
    } else
        kDebug() << "Not possible to stat, skipped.";
}

void PluginBookmarkSync::slotConfigure()
{
    // dialog
    m_settings->exec();

    // reset the actions and bools.
    m_storeAction->setEnabled( m_location.isValid() );
    m_fetchAction->setEnabled( m_location.isValid() );
    m_autoSave = m_settings->AutoSave->isChecked();
    m_autoFetch = m_settings->AutoFetch->isChecked();

    // start or stop the watch if needed.
    if ( m_autoSave && m_watch->isStopped() )
        m_watch->startScan();
    else if ( !m_autoSave && !m_watch->isStopped() )
        m_watch->stopScan();

    // save the settings...
    KConfigGroup group = m_config->group( "Settings" );
    group.writeEntry( "Location", m_location );
    group.writeEntry( "AutoSave", m_autoSave );
    group.writeEntry( "AutoFetch", m_autoFetch );
    m_config->sync();
}

void PluginBookmarkSync::slotGetLocation()
{
    // Presents the file dialog to the user where he can pick a location.
    KUrl url = m_location.isValid() ? m_location : KUrl();
    KUrl location = KFileDialog::getExistingDirectoryUrl( url, m_parent ,i18n( "Please indicate (remote) location" ) );
    if ( location.isValid() ) {
        m_location = location;
        m_settings->kpushbutton->setText( m_location.prettyUrl() );
    }
}

void PluginBookmarkSync::copy( Type type )
{
    // prepare the KUrl for the local bookmarks.
    KUrl other;
    QString otherFile = KStandardDirs::locateLocal( "data", "konqueror/bookmarks.xml" );
    if ( KStandardDirs::exists( otherFile ) )
        other = KUrl( otherFile );
    else if ( type == Save ) {
        KMessageBox::error( m_parent, i18n( "No bookmarks file found" ) );
        return;
    }

    // prepare the KUrl for the remote bookmarks.
    KUrl bookmarks( m_location );
    bookmarks.addPath( "bookmarks.xml" );

    // Start the copy.
    if ( type == Save ) {
        kDebug() << "Copying " << other << "to" << bookmarks;
        KIO::FileCopyJob* job = KIO::file_copy( other, bookmarks, -1, KIO::Overwrite );
        connect( job, SIGNAL( result( KJob* ) ), SLOT( slotDone( KJob* ) ) );
    } else if ( type == Fetch ) {
        kDebug() << "Copying " << bookmarks << "to" << other;
        KIO::FileCopyJob* job = KIO::file_copy( bookmarks, other, -1, KIO::Overwrite );
        connect( job, SIGNAL( result( KJob* ) ), SLOT( slotDone( KJob* ) ) );
    }
}

void PluginBookmarkSync::slotDone( KJob* job )
{
    if ( job->error() > 0 )
        KMessageBox::error( m_parent, i18n( "Sync failed: %1", job->errorString() ) );
    else {
        // Refresh the bookmarks in konqueror.
        KBookmarkManager* mgr = KBookmarkManager::userBookmarksManager();
        mgr->notifyCompleteChange( "bookmarksync" );
        m_localSize = getLocalSize();
    }

    m_autoFetchInProgress = false;

    if ( m_autoSave && m_watch->isStopped() ) // restart it if needed.
        m_watch->startScan();
}

void PluginBookmarkSync::slotSave()
{
    kDebug() << "Going to save in 60 seconds";
    // delay it for 60 seconds to catch all changes.
    m_saveTimer->start( 60 * 1000 );
}

void PluginBookmarkSync::slotSaveDoIt()
{
    kDebug() << "Going to save now";
    copy( Save );
}

void PluginBookmarkSync::slotSaveMaybe()
{
    // sometimes konqi saves without a change. No need to upload in that case.
    int localSize = getLocalSize();
    kDebug() << "Orig size: " << m_localSize << "New size" << localSize;
    if ( m_localSize != localSize ) {
        m_localSize = localSize;
        slotSave();
    } else
        kDebug() << "No change, so ignore this...";
}

int PluginBookmarkSync::getLocalSize()
{
    KIO::UDSEntry entryHere;
    KUrl bookmarksHere( KStandardDirs::locateLocal( "data", "konqueror/bookmarks.xml" ) );
    bool okHere = KIO::NetAccess::stat( bookmarksHere, entryHere, m_parent );
    if ( okHere )
        return entryHere.numberValue( KIO::UDSEntry::UDS_SIZE, -1 );
    return -1;
}

void PluginBookmarkSync::slotFetch()
{
    if ( !m_autoFetchInProgress && m_watch && m_autoSave && !m_watch->isStopped() )
        m_watch->stopScan(); // prevent an AutoSave cycle ;-)
    copy( Fetch );
}

#include "plugin_bookmarksync.moc"
