/* This file is part of the Nepomuk Project
   Copyright (c) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "datamaintenanceservice.h"

#include "nie.h"
#include "nfo.h"
#include "nuao.h"

#include "usageevent.h"
#include "modificationevent.h"

#include <KUrl>
#include <KLocale>
#include <KPluginFactory>
#include <KDebug>
#include <KSharedConfig>
#include <KConfigGroup>

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include <Nepomuk/Query/FileQuery>

#include <QtCore/QRegExp>
#include <QtCore/QThread>

#include <Soprano/Node>
#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>

#include <math.h>


namespace {
    QUrl scoringGraphUri() {
        return QUrl::fromEncoded( "urn:nepomuk:statisticaldata", QUrl::StrictMode );
    }
}


class Nepomuk::DataMaintenanceService::ScoringThread : public QThread
{
public:
    ScoringThread( DataMaintenanceService* parent );
    ~ScoringThread();

    void run();

    void cancel() {
        m_canceled = true;
    }

private:
    DataMaintenanceService* m_service;
    bool m_canceled;
};


Nepomuk::DataMaintenanceService::ScoringThread::ScoringThread( DataMaintenanceService* parent )
    : QThread( parent ),
      m_service( parent )
{
}


Nepomuk::DataMaintenanceService::ScoringThread::~ScoringThread()
{
}


void Nepomuk::DataMaintenanceService::ScoringThread::run()
{
    kDebug();

    // preparation
    m_canceled = false;
    m_service->rebuildFilenamePatternWeightRxCache();

    // TODO: make this property with metadata graph and maybe even a random uri
    const QUrl scoringContext = scoringGraphUri();

    // clear out old scores
    m_service->mainModel()->removeContext( scoringContext );

    // remember current time to have a stable value of comparison for all files
    const QDateTime now = QDateTime::currentDateTime();

    // empty files query to list all files
    Query::FileQuery query;
    query.setFileMode( Query::FileQuery::QueryFiles );

    Soprano::QueryResultIterator it = m_service->mainModel()->executeQuery( query.toSparqlQuery(),
                                                                            Soprano::Query::QueryLanguageSparql );
    while ( !m_canceled && it.next() ) {
        Resource res = Resource::fromResourceUri( it[0].uri() );
        double score = m_service->calculateScore( res, now );
        m_service->mainModel()->addStatement( res.resourceUri(),
                                              Soprano::Vocabulary::NAO::score(),
                                              Soprano::LiteralValue(score),
                                              scoringContext );
    }
    kDebug() << now.secsTo( QDateTime::currentDateTime() );
}




Nepomuk::DataMaintenanceService::DataMaintenanceService( QObject* parent, const QList<QVariant>& )
    : Service( parent ),
      m_scoringThread( new ScoringThread( this ) )
{
    // make sure we do not init in the ScoringThread
    mainModel();
}


Nepomuk::DataMaintenanceService::~DataMaintenanceService()
{
    m_scoringThread->cancel();
    m_scoringThread->wait();
}


void Nepomuk::DataMaintenanceService::resourceOpened( const QString& uri, const QString& service )
{
    if ( findPendingUsageEvent( uri, service, false ) == m_pendingEvents.end() ) {
        m_pendingEvents.append( createUsageEvent( uri, service ) );
    }
    else {
        kDebug() << "Ignoring duplicate event:" << uri << service;
    }
}


void Nepomuk::DataMaintenanceService::resourceOpenedOneShot( const QString& uri, const QString& service )
{
    createUsageEvent( uri, service );
}


void Nepomuk::DataMaintenanceService::resourceSaved( const QString& uri, const QString& service )
{
    QList<UsageEventContainer>::iterator it = findPendingUsageEvent( uri, service, true );
    if ( it != m_pendingEvents.end() ) {
        UsageEventContainer& uec = *it;
        uec.event.setEnd( QDateTime::currentDateTime() );
        uec.event.addType( Nepomuk::Vocabulary::NUAO::ModificationEvent() );
        m_pendingEvents.erase( it );
    }
}


void Nepomuk::DataMaintenanceService::resourceClosed( const QString& uri, const QString& service )
{
    QList<UsageEventContainer>::iterator it = findPendingUsageEvent( uri, service, true );
    if ( it != m_pendingEvents.end() ) {
        UsageEventContainer& uec = *it;
        uec.event.setEnd( QDateTime::currentDateTime() );
        m_pendingEvents.erase( it );
    }
}


double Nepomuk::DataMaintenanceService::scoreForResource( const QString& uri )
{
    rebuildFilenamePatternWeightRxCache();
    return calculateScore( Resource( KUrl( uri ) ), QDateTime::currentDateTime() );
}


// TODO: make this async by using a queue of resources to update
void Nepomuk::DataMaintenanceService::updateResourceScore( const QString& uri )
{
    double score = scoreForResource( uri );
    KUrl resourceUri( uri );
    mainModel()->removeAllStatements( resourceUri,
                                      Soprano::Vocabulary::NAO::score(),
                                      Soprano::Node(),
                                      scoringGraphUri() );
    mainModel()->addStatement( resourceUri,
                               Soprano::Vocabulary::NAO::score(),
                               Soprano::LiteralValue(score),
                               scoringGraphUri() );
}


void Nepomuk::DataMaintenanceService::updateFileScores()
{
    if ( !m_scoringThread->isRunning() )
        m_scoringThread->start();
}


void Nepomuk::DataMaintenanceService::rebuildFilenamePatternWeightRxCache()
{
    m_filenamePatternWeightRxCache.clear();

    KSharedConfigPtr typeWeightConfig = KSharedConfig::openConfig( "nepomuktypeweightsrc" );
    KConfigGroup grp = typeWeightConfig->group( "filename pattern weights" );
    const QStringList patterns = grp.keyList();

    Q_FOREACH( const QString& pattern, patterns ) {
        QRegExp rx( pattern );
        rx.setPatternSyntax( QRegExp::Wildcard );
        m_filenamePatternWeightRxCache.append( qMakePair( rx, grp.readEntry( pattern, 1.0 ) ) );
    }
}


double Nepomuk::DataMaintenanceService::lookupTypeWeight( const KUrl& typeUri ) const
{
    KSharedConfigPtr typeWeightConfig = KSharedConfig::openConfig( "nepomuktypeweightsrc" );
    KConfigGroup grp = typeWeightConfig->group( "class weights" );

    if ( grp.hasKey( typeUri.url() ) ) {
        return grp.readEntry( typeUri.url(), 1.0 );
    }
    else {
        // we do a breadth-first search
        const Nepomuk::Types::Class type( typeUri );
        QList<Types::Class> parentTypes = type.allParentClasses();
        Q_FOREACH( const Types::Class& parentType, parentTypes ) {
            const KUrl parentTypeUri( parentType.uri() );
            if ( grp.hasKey( parentTypeUri.url() ) ) {
                return grp.readEntry( parentTypeUri.url(), 1.0 );
            }
        }

        Q_FOREACH( const Types::Class& parentType, parentTypes ) {
            const double score = lookupTypeWeight( parentType.uri() );
            if ( score != 1.0 ) {
                return score;
            }
        }
    }

    // fallback
    return 1.0;
}


double Nepomuk::DataMaintenanceService::lookupFilenamePatternWeight( const QString& filename ) const
{
    for ( int i = 0; i < m_filenamePatternWeightRxCache.count(); ++i ) {
        if ( m_filenamePatternWeightRxCache[i].first.exactMatch( filename ) ) {
//            kDebug() << filename << m_filenamePatternWeightRxCache[i].first.pattern() << m_filenamePatternWeightRxCache[i].second;
            return m_filenamePatternWeightRxCache[i].second;
        }
    }
    // fallback
    return 1.0;
}


namespace {
    double diffDates( const QDateTime& from, const QDateTime& to )
    {
        int diff = from.secsTo( to );
        return double(diff)/60.0/60.0/1000.0;
    }

    double accessDateWeight( const QDateTime& date, const QDateTime& now ) {
        double diff = diffDates( date, now );
        if ( diff < 0 )
            return 0.0;
        else
            return ::exp( -1.0 * diff );
    }

    /**
     * Give a linear weighting based on the rating. No rating is treated
     * as 5.5 - average. This needs to be optimized so its impact is not too
     * big.
     */
    double ratingWeight( int rating ) {
        rating = qMax( 0, qMin( 10, rating ) );
        double normalizedRating = double( rating );
        if ( rating == 0 )
            normalizedRating = 5.5;
        return normalizedRating / 20.0;
    }
}

double Nepomuk::DataMaintenanceService::calculateScore( const Nepomuk::Resource& res, const QDateTime& now )
{
//    kDebug() << res.resourceUri();

    // we use a base score of 1 to never have scores = 0
    double score = 1.0;

    //
    // Sum the weight of all open events using an exponential function to give less
    // weight to old open events.
    //
    int openEventCnt = 0;
    const QString query = QString::fromLatin1( "select ?date where { "
                                               "?r a ?t . ?t rdfs:subClassOf nuao:DesktopEvent . "
                                               "?r nuao:involves %1 . "
                                               "?r nuao:start ?date . }" )
                          .arg( Soprano::Node::resourceToN3( res.resourceUri() ) );
    Soprano::QueryResultIterator it = mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        QDateTime date = it[QLatin1String( "date" )].literal().toDateTime();
        score += accessDateWeight( date, now );
        ++openEventCnt;
    }


    //
    // not all usage events have an actual DesktopEvent resource
    // So we calculate them in somehow
    //
    if ( res.usageCount() - openEventCnt > 0 ) {
        // FIXME: do something with cnt
    }


    //
    // the last modification date is slightly more important than the usage events
    // if a file has never been used it is even more important than those that have been
    // used a few times
    //
    QDateTime modDate;
    if ( res.hasProperty( Nepomuk::Vocabulary::NIE::lastModified() ) ) {
        modDate = res.property( Nepomuk::Vocabulary::NIE::lastModified() ).toDateTime();
    }
    else {
        modDate = res.property( Soprano::Vocabulary::NAO::lastModified() ).toDateTime();
    }
    // TODO: if openEventCnt == 0 -> give this a higher weight
    const double modWeight = 1.2 * accessDateWeight( modDate, now );
//    kDebug() << "modification date score:" << modDate << modWeight;
    score += modWeight;


    //
    // the rating also changes the final score
    // A high rating increases the score while a low rating decreases it
    // (here no rating is better than a low rating)
    //
//    kDebug() << "Rating weight:" << ratingWeight( res.rating() );
    score *= ratingWeight( res.rating() );


    //
    // modify the score based on the resource type
    //
//    kDebug() << "Type weight:" << lookupTypeWeight( res.resourceType() );
    score *= lookupTypeWeight( res.resourceType() );


    //
    // Factor in filename patterns which give a finer grained selection to some
    // type weights like nfo:PlainTextDocument where *.log is less interesting
    // than *.txt
    //
    if ( res.hasType( Nepomuk::Vocabulary::NFO::FileDataObject() ) ) {
        const QString filename = KUrl( res.property( Nepomuk::Vocabulary::NIE::url() ).toUrl() ).fileName();
        score *= lookupFilenamePatternWeight( filename );
    }

//    kDebug() << "final score:" << score;

    return score;
}


Nepomuk::DataMaintenanceService::UsageEventContainer Nepomuk::DataMaintenanceService::createUsageEvent( const QString& uri, const QString& service ) const
{
    UsageEventContainer uec;
    uec.resource = KUrl( uri );
    uec.timestamp = QDateTime::currentDateTime();
    uec.event.setStart( uec.timestamp );
    uec.event.addInvolves( uec.resource );
    uec.resource.increaseUsageCount();
    uec.service = service;
    // TODO: relate event to service
    return uec;
}


QList<Nepomuk::DataMaintenanceService::UsageEventContainer>::iterator Nepomuk::DataMaintenanceService::findPendingUsageEvent( const QString& uri,
                                                                                                                              const QString& service,
                                                                                                                              bool ignoreTimestamp )
{
    const QDateTime now = QDateTime::currentDateTime();
    const Nepomuk::Resource resource = KUrl( uri );
    QList<UsageEventContainer>::iterator it = m_pendingEvents.begin();
    while ( it != m_pendingEvents.end() ) {
        UsageEventContainer& uec = *it;
        if ( resource == uec.resource &&
             service == uec.service &&
             ( ignoreTimestamp ||
               uec.timestamp.secsTo( now ) < 10000 ) ) {
            return it;
        }
        ++it;
    }
    return m_pendingEvents.end();
}

NEPOMUK_EXPORT_SERVICE( Nepomuk::DataMaintenanceService, "nepomukdatamanagementservice")

#include "datamaintenanceservice.moc"
