/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QDateTime>
#include <QString>

#include <KDebug>

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

#include "filesystem.h"
#include "nsvo.h"

namespace Nepomuk
{
    // there should be a vocabulary class for NFO in either Soprano or Nepomuk
    const QUrl filesystemUrl( "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#FileSystem" );

    Filesystem::Filesystem( const QUrl& identifier, const QString& mountpoint, const QString& label, bool mounted, qulonglong byteSize, int priority )
    {
        d = new FilesystemData;
        d->isValid = true;
        d->identifer = identifier;
        d->mountpoint =  mountpoint;
        d->label = label;
        d->mounted = mounted;
        d->byteSize = byteSize;
        d->priority = priority;
        if (mounted) {
            d->lastmounted = QDateTime::currentDateTime();
        }
    }


    Filesystem::Filesystem(const Filesystem &other)
            : d( other.d )
    {
    }


    void Filesystem::setMounted( bool mounted, const QString& mountpoint )
    {
        Q_ASSERT( d->isValid );
        d->mounted = mounted;
        // also set this for unmounts, because we want to store the time when the fs was last available
        d->lastmounted = QDateTime::currentDateTime();
        if ( mounted ) {
           d->mountpoint = mountpoint;
       }
    }


    QString Filesystem::mountpoint() const {
        Q_ASSERT( d->isValid );
        return d->mountpoint;
    }

    int Filesystem::priority() const {
        Q_ASSERT( d->isValid );
        return d->priority;
    }

    bool Filesystem::writeToNepomuk()
    {
        Q_ASSERT( d->isValid );
        // resource not valid means, this filesystem has not been written yet, so we have to write all of it
        if ( !d->resource.isValid() )
        {
            // new Filesystems are only written when mounted, to be able to  match against mountpoint
            // which is needed to match filesystems from other filesystemhandlers which create different identifiers

            if ( !d->mounted )
            {
                kDebug() << "Tried to write an unmounted filesystem to Nepomuk, only state update allowed when unmounted";
                return false;
            }

            initNepomukResource();

            d->resource.setProperty( Nepomuk::Vocabulary::NSVO::fileSystemIdentifier(), Variant( d->identifer ) );
            d->resource.setLabel( d->label );
            if ( d->byteSize ) {
                d->resource.setProperty( Nepomuk::Vocabulary::NSVO::byteSize(), Variant ( d->byteSize )  );
            }

        }
        // only update mountstate
        d->resource.setProperty( Nepomuk::Vocabulary::NSVO::isMounted(), Variant( d->mounted ) );
        d->resource.setProperty( Nepomuk::Vocabulary::NSVO::lastMountTime(), Variant( d->lastmounted) );

        if ( d->mounted ) {
            d->resource.setProperty( Nepomuk::Vocabulary::NSVO::lastMountedOn(), Variant( d->mountpoint ) );
        }

        return true;
    }


    void Filesystem::initNepomukResource()
    {
        Q_ASSERT( d->isValid );
        // try to find the filesystem with the same identifer
        QList< Nepomuk::Resource > resources = Nepomuk::ResourceManager::instance()->allResourcesWithProperty( Nepomuk::Vocabulary::NSVO::fileSystemIdentifier() , Variant( d->identifer ) );

        if ( !resources.isEmpty() ) {
            d->resource = resources.first();
            return;
        } else {
            // generate new resource
            d->resource = Nepomuk::Resource( Nepomuk::ResourceManager::instance()->generateUniqueUri(),  filesystemUrl );
        }

    }

}
