/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <Solid/Device>
#include <Solid/DeviceNotifier>
#include <Solid/StorageDrive>
#include <Solid/StorageVolume>
#include <Solid/StorageAccess>


#include "solidfilesystemhandler.h"
#include "filesystem.h"

namespace {
        bool isUsableVolume( const Solid::Device& dev ) {
        if ( dev.is<Solid::StorageVolume>() &&
             dev.is<Solid::StorageAccess>() &&
             dev.parent().is<Solid::StorageDrive>() ) {
            const Solid::StorageVolume* volume = dev.as<Solid::StorageVolume>();
            if ( volume->usage() == Solid::StorageVolume::FileSystem )
                return true;
        }
        return false;
    }
}


namespace Nepomuk
{

SolidFilesystemHandler::SolidFilesystemHandler(QObject *parent, int priority )
        : AbstractFilesystemHandler( parent ), m_priority( priority )

{
    initCacheEntries();
    connect( Solid::DeviceNotifier::instance(), SIGNAL( deviceAdded( const QString& ) ),
             this, SLOT( slotSolidDeviceAdded( const QString& ) ) );
    connect( Solid::DeviceNotifier::instance(), SIGNAL( deviceRemoved( const QString& ) ),
             this, SLOT( slotSolidDeviceRemoved( const QString& ) ) );
}

QList<Filesystem> SolidFilesystemHandler::mountedFilesystems() const
{
    return m_mountedFilesystems.values();
}


void SolidFilesystemHandler::initCacheEntries()
{
    QList<Solid::Device> devices
        = Solid::Device::listFromQuery( QLatin1String( "StorageVolume.usage=='FileSystem'" ) );
    foreach( const Solid::Device& dev, devices ) {
        if ( isUsableVolume( dev ) ) {
            createCacheEntry( dev );
        }
    }
}

const Filesystem SolidFilesystemHandler::filesystemForEntry( const Entry& entry ) const
{
    QUrl identifer;
    identifer.setScheme( "nepomuk_solid" );
    identifer.setPath( entry.m_udi );

    QString label = entry.m_storageVolume->label();
    bool mounted = entry.m_storageAccess->isAccessible();
    QString mountpoint = mounted ? entry.m_storageAccess->filePath() : QString();
    qulonglong byteSize = entry.m_storageVolume->size();

    Filesystem filesystem( identifer, mountpoint, label, mounted, byteSize, m_priority );

    return filesystem;
}

void SolidFilesystemHandler::slotSolidDeviceAdded( const QString& udi )
{
    Solid::Device dev = Solid::Device( udi );
    if ( isUsableVolume( dev ) ) {
        createCacheEntry( dev );
    }
}


void SolidFilesystemHandler::slotSolidDeviceRemoved( const QString& udi )
{
    if ( m_metadataCache.contains( udi ) ) {
         m_metadataCache.remove( udi );
    }
    m_mountedFilesystems.remove( udi );
}


void SolidFilesystemHandler::slotAccessibilityChanged( bool accessible, const QString& udi )
{
    Q_ASSERT( m_metadataCache.contains( udi ) );
    Filesystem fs = m_metadataCache[udi].m_filesystem;
    QString mountpoint = accessible ? m_metadataCache[udi].m_storageAccess->filePath() : QString();

    fs.setMounted( accessible, mountpoint );
    if ( accessible ) {
        m_mountedFilesystems.insert( udi, fs );
        emit filesystemMounted( fs );
    } else {
        m_mountedFilesystems.remove( udi );
        emit filesystemUnmounted( fs );
    }
}


SolidFilesystemHandler::Entry::Entry( SolidFilesystemHandler* parent )
    : q( parent )
{
}


SolidFilesystemHandler::Entry* SolidFilesystemHandler::createCacheEntry( const Solid::Device& dev )
{
    Entry entry( this );
    entry.m_storageAccess = dev.as<Solid::StorageAccess>();
    entry.m_storageVolume = dev.as<Solid::StorageVolume>();
    entry.m_udi = dev.udi();

    connect( entry.m_storageAccess, SIGNAL(accessibilityChanged(bool, QString)),
         this, SLOT(slotAccessibilityChanged(bool, QString)) );

    entry.m_filesystem = this->filesystemForEntry( entry );
    Q_ASSERT( entry.m_filesystem.isValid() );
    m_metadataCache.insert( dev.udi(), entry );


    if ( entry.m_storageAccess->isAccessible() ) {
        this->m_mountedFilesystems.insert( dev.udi(), entry.m_filesystem );
    }

    return &m_metadataCache[dev.udi()];
}

}


#include "solidfilesystemhandler.moc"
