/*
 * Copyright (c) 2010 Ivan Cukic <ivan.cukic(at)kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "test-activity-manager.h"
#include <KDebug>
#include <KFileDialog>
#include <KWindowInfo>
#include <KWindowSystem>

#include <QDBusConnection>
#define tDebug(A) \
    textDebug->setPlainText(A + "\n" + textDebug->toPlainText());

TestActivityManager::TestActivityManager()
    : TAMWindowBase()
{
    setupUi(this);

    // resize(200, 200);

    m_controller = new KActivityController();

    connect(m_controller, SIGNAL(activityAdded(const QString &)),
            this, SLOT(activityAdded(const QString &)));

    connect(m_controller, SIGNAL(activityRemoved(const QString &)),
            this, SLOT(activityRemoved(const QString &)));

    connect(m_controller, SIGNAL(resourceWindowRegistered(const QString &, uint, const QString &)),
            this, SLOT(resourceWindowRegistered(const QString &, uint, const QString &)));

    connect(m_controller, SIGNAL(resourceWindowUnregistered(uint, const QString &)),
            this, SLOT(resourceWindowUnregistered(uint, const QString &)));

    connect(m_controller, SIGNAL(currentActivityChanged(const QString &)),
            this, SLOT(currentActivityChanged(const QString &)));

    connect(buttonActivityLoadData, SIGNAL(clicked()),
            this, SLOT(buttonActivityLoadDataClicked()));

    connect(buttonListAssignedDocuments, SIGNAL(clicked()),
            this, SLOT(buttonListAssignedDocumentsClicked()));

    connect(buttonListAssignedFolders, SIGNAL(clicked()),
            this, SLOT(buttonListAssignedFoldersClicked()));

    connect(buttonListAssignedResources, SIGNAL(clicked()),
            this, SLOT(buttonListAssignedResourcesClicked()));

    connect(buttonAssignDocument, SIGNAL(clicked()),
            this, SLOT(buttonAssignDocumentClicked()));

    connect(buttonAssignFolder, SIGNAL(clicked()),
            this, SLOT(buttonAssignFolderClicked()));

    connect(buttonAssignResource, SIGNAL(clicked()),
            this, SLOT(buttonAssignResourceClicked()));

    updateActivitiesList();
}

TestActivityManager::~TestActivityManager()
{
    delete m_controller;
}

void TestActivityManager::updateActivitiesList()
{
    QString contents;

    foreach (const QString & activity, m_controller->availableActivities()) {
        contents += activity + "\n Name: "
                 + (new KActivityInfo(activity, this))->name() + "\n";
    }
    textActivities->setPlainText(contents);
}

void TestActivityManager::activityAdded(const QString & id)
{
    tDebug("activityAdded: " + id);
    updateActivitiesList();
}

void TestActivityManager::activityRemoved(const QString & id)
{
    tDebug("activityRemoved: " + id);
    updateActivitiesList();
}

void TestActivityManager::resourceWindowRegistered(const QString & application, uint wid, const QString & uri)
{
    tDebug("resourceWindowRegistered: " + application + " - " + QString::number(wid) + " " + uri);

    // KWindowInfo info = KWindowSystem::windowInfo(wid,
    //     NET::WMName | NET::WMVisibleName | NET::WMWindowType,
    //     NET::NET::WM2WindowClass | NET::WM2WindowRole
    //     );

    // tDebug("  vvv  " + info.name()
    //      + "#" + info.visibleName()
    //      + "#" + info.windowClassClass()
    //      + "#" + info.windowClassName()
    //      + "#" + info.windowRole()
    // );

    updateActivitiesList();
}

void TestActivityManager::resourceWindowUnregistered(uint wid, const QString & uri)
{
    tDebug("resourceWindowUnregistered: " + QString::number(wid) + " " + uri);
    updateActivitiesList();
}

void TestActivityManager::currentActivityChanged(const QString & id)
{
    tDebug("currentActivityChanged: " + id);
    updateActivitiesList();
}

void TestActivityManager::activityNameChanged(const QString & id, const QString & name)
{
    tDebug("activityNameChanged: " + id + " " + name);
    updateActivitiesList();
}

void TestActivityManager::buttonActivityLoadDataClicked()
{
    m_info = new KActivityInfo(editActivity->text(), this);

    if (!m_info) {
        textActivity->setText("NULL");
        return;
    }

    QStringList lines;

    lines
        << "url:" + m_info->uri().url()
        << "resource url:" + m_info->resourceUri().url();

    textActivity->setText(lines.join("\n"));
}

void TestActivityManager::buttonListAssignedDocumentsClicked()
{
    buttonActivityLoadDataClicked();
    if (!m_info) return;

    QStringList items;
    items << "docs:";

    foreach (const KUrl & url, m_info->associatedResources(KActivityInfo::DocumentResource)) {
        items << url.prettyUrl();
    }

    textActivity->setText(textActivity->toPlainText() + "\n" + items.join("\n"));
}

void TestActivityManager::buttonListAssignedFoldersClicked()
{
    buttonActivityLoadDataClicked();
    if (!m_info) return;

    QStringList items;
    items << "folders:";

    foreach (const KUrl & url, m_info->associatedResources(KActivityInfo::FolderResource)) {
        items << url.prettyUrl();
    }

    textActivity->setText(textActivity->toPlainText() + "\n" + items.join("\n"));
}

void TestActivityManager::buttonListAssignedResourcesClicked()
{
    buttonActivityLoadDataClicked();
    if (!m_info) return;

    QStringList items;
    items << "all resources:";

    foreach (const KUrl & url, m_info->associatedResources()) {
        items << url.prettyUrl();
    }

    textActivity->setText(textActivity->toPlainText() + "\n" + items.join("\n"));
}

void TestActivityManager::buttonAssignDocumentClicked()
{
    textActivity->setText("");

    m_info = new KActivityInfo(editActivity->text(), this);

    if (!m_info) {
        textActivity->setText("NULL");
        return;
    }

    QString file = KFileDialog::getOpenFileName();

    if (file.isEmpty()) return;

    m_info->associateResource(KUrl(file), KActivityInfo::DocumentResource);

}

void TestActivityManager::buttonAssignFolderClicked()
{
    textActivity->setText("");

    m_info = new KActivityInfo(editActivity->text(), this);

    if (!m_info) {
        textActivity->setText("NULL");
        return;
    }

    QString file = KFileDialog::getOpenFileName();

    if (file.isEmpty()) return;

    m_info->associateResource(KUrl(file), KActivityInfo::FolderResource);


}

void TestActivityManager::buttonAssignResourceClicked()
{

}



#include "test-activity-manager.moc"
