/*
 * Copyright (c) 2010 Ivan Cukic <ivan.cukic(at)kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "activitymanager.h"
#include "activitymanager_p.h"
#include "activitymanageradaptor.h"

//#include "nepomukactivitiesservice_interface.h"
//#include "activitycontroller_interface.h"

#include <stdint.h>

#include <QUuid>
#include <QDBusConnection>
#include <QDBusConnectionInterface>
#include <QDBusServiceWatcher>

#include <KPluginFactory>
#include <KPluginLoader>
#include <KConfig>
#include <KConfigGroup>
#include <KUrl>
#include <KDebug>

#include <KWindowInfo>
#include <KWindowSystem>

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>

#define ActivityManagerServicePath "org.kde.ActivityManager"
#define ACTIVITIES_PROTOCOL "activities://"

K_PLUGIN_FACTORY(ActivityManagerFactory,
                 registerPlugin<ActivityManager>();)
K_EXPORT_PLUGIN(ActivityManagerFactory("activitymanager"))

// Private

ActivityManager::Private::Private()
    : config("activitymanagerrc")
{
    // Initializing config
    connect(&configSyncTimer, SIGNAL(timeout()),
             this, SLOT(configSync()));

    configSyncTimer.setSingleShot(true);
    configSyncTimer.setInterval(2 * 60 * 1000);
}

KConfigGroup ActivityManager::Private::activitiesConfig()
{
    return KConfigGroup(&config, "activities");
}

KConfigGroup ActivityManager::Private::mainConfig()
{
    return KConfigGroup(&config, "main");
}

QString ActivityManager::Private::activityName(const QString & id)
{
    return activitiesConfig().readEntry(id, QString());
}

void ActivityManager::Private::scheduleConfigSync()
{
    if (!configSyncTimer.isActive()) {
        configSyncTimer.start();
    }
}

void ActivityManager::Private::configSync()
{
    configSyncTimer.stop();
    config.sync();
}

Nepomuk::Resource ActivityManager::Private::activityResource(const QString & id) const
{
    return Nepomuk::Resource(KUrl(ACTIVITIES_PROTOCOL + id));
}

// ActivityManager

ActivityManager::ActivityManager(QObject * parent, const QList < QVariant > & )
    : KDEDModule(parent), d(new Private())
{
    // Registering DBus service
    QDBusConnection conn = QDBusConnection::sessionBus();

    if (conn.interface()->isServiceRegistered(ActivityManagerServicePath)) {
        kError() << "already running";
        return;
    }

    new ActivityManagerAdaptor(this);
    conn.interface()->registerService(ActivityManagerServicePath);
    conn.registerObject("/ActivityManager", this);

    // Initializing Nepomuk
    Nepomuk::ResourceManager::instance()->init();

    // Reading config
    d->availableActivities = d->activitiesConfig().keyList();
    d->runningActivities = d->mainConfig().readEntry("runningActivities", d->availableActivities);

    d->currentActivity = d->mainConfig().readEntry("currentActivity", QString());
    if (!d->availableActivities.contains(d->currentActivity)) {
         d->currentActivity.clear();
    }
}

ActivityManager::~ActivityManager()
{
    d->config.sync();
    delete d;
}

QString ActivityManager::CurrentActivity() const
{
    return d->currentActivity;
}

bool ActivityManager::SetCurrentActivity(const QString & id)
{
    if (!d->availableActivities.contains(id)) {
        return false;
    }

    d->currentActivity = id;
    d->mainConfig().writeEntry("currentActivity", id);

    d->scheduleConfigSync();

    emit CurrentActivityChanged(id);
    return true;
}

QString ActivityManager::AddActivity(const QString & name)
{
    QString id;

    // Ensuring a new Uuid. The loop should usually end after only
    // one iteration
    while (id.isEmpty() || d->availableActivities.contains(id)) {
        id = QUuid::createUuid();
        id.replace(QRegExp("[{}]"), QString());
    }

    d->availableActivities << id;
    d->runningActivities << id;

    SetActivityName(id, name);

    emit ActivityAdded(id);

    d->configSync();
    return id;
}

void ActivityManager::StashActivity(const QString & id)
{
    if (!d->runningActivities.contains(id)) return;

    d->runningActivities.removeAll(id);
    d->mainConfig().writeEntry("runningActivities", d->runningActivities);

    emit ActivityStashed(id);
    d->configSync();
}

void ActivityManager::RemoveActivity(const QString & id)
{
    if (d->availableActivities.size() < 2 ||
            !d->availableActivities.contains(id)) {
        return;
    }

    // If the activity is sunning, stash it
    StashActivity(id);

    // Removing the activity
    d->availableActivities.removeAll(id);
    d->activitiesConfig().deleteEntry(id);

    // If the removed activity was the current one,
    // set another activity as current
    if (d->currentActivity == id) {
        SetCurrentActivity(d->availableActivities.first());
    }

    emit ActivityRemoved(id);
    d->configSync();
}

QStringList ActivityManager::AvailableActivities() const
{
    return d->availableActivities;
}

QStringList ActivityManager::RunningActivities() const
{
    return d->runningActivities;
}

QStringList ActivityManager::StashedActivities() const
{
    return (d->availableActivities.toSet() - d->runningActivities.toSet()).toList();
}

QString ActivityManager::ActivityName(const QString & id) const
{
    return d->activityName(id);
}

void ActivityManager::SetActivityName(const QString & id, const QString & name)
{
    if (!d->availableActivities.contains(id)) {
        return;
    }

    d->activitiesConfig().writeEntry(id, name);

    if (Nepomuk::ResourceManager::instance()->initialized()) {
        d->activityResource(id).setLabel(name);
    }

    d->scheduleConfigSync();
    emit ActivityChanged(id);
}

QString ActivityManager::ActivityIcon(const QString & id) const
{
    if (!d->availableActivities.contains(id) ||
            !Nepomuk::ResourceManager::instance()->initialized()) {
        return QString();
    }

    QStringList symbols = d->activityResource(id).symbols();

    if (symbols.isEmpty()) {
        return QString();
    } else {
        return symbols.first();
    }
}

void ActivityManager::SetActivityIcon(const QString & id, const QString & icon)
{
    if (!d->availableActivities.contains(id) ||
            !Nepomuk::ResourceManager::instance()->initialized()) {
        return;
    }

    d->activityResource(id).setSymbols(QStringList() << icon);

    emit ActivityChanged(id);
}

QString ActivityManager::ActivityDescription(const QString & id) const
{
    if (!d->availableActivities.contains(id) ||
            !Nepomuk::ResourceManager::instance()->initialized()) {
        return QString();
    }

    return d->activityResource(id).description();
}

void ActivityManager::SetActivityDescription(const QString & id, const QString & description)
{
    if (!d->availableActivities.contains(id) ||
            !Nepomuk::ResourceManager::instance()->initialized()) {
        return;
    }

    d->activityResource(id).setDescription(description);

    emit ActivityChanged(id);
}

void ActivityManager::NotifyResourceAccessed(const QString & application, const QString & uri)
{
    // Nothing at the moment
}

void ActivityManager::NotifyResourceOpened(const QString & application, uint windowId, const QString & uri)
{
    if (!d->applications.contains(windowId)) {
        d->applications[windowId] = application;
    }

    KUrl kuri(uri);
    d->resources[windowId] << kuri;
    d->activities[kuri] << CurrentActivity();
}

void ActivityManager::NotifyResourceModified(uint windowId, const QString & uri)
{
    // Nothing at the moment
}

void ActivityManager::NotifyResourceClosed(uint windowId, const QString & uri)
{
    d->resources[windowId].remove(KUrl(uri));

    if (d->resources[windowId].size() == 0) {
        d->resources.remove(windowId);
        d->applications.remove(windowId);
    }

}

QStringList ActivityManager::ActivitiesForResource(const QString & uri)
{
    QSet < QString > result = d->activities.value(uri);

    // if (d->activitiesStore) {
    //     QStringList fromStore = d->activitiesStore->forResource(uri);
    //     result += QSet < QString > ::fromList(fromStore);
    // }

    return result.toList();


}


// Temporary functions

QString ActivityManager::_allInfo() const
{
    QStringList result;

    // {
    //     QHashIterator < QString, QSet < QString > > i (d->resourceActivities);
    //     while (i.hasNext()) {
    //         i.next();

    //         QStringList list = i.value().toList();
    //         result << i.key() + ':' + list.join(" ") + '\n';
    //     }
    // }

    // {
    //     QHashIterator < WId, QSet < QString > > i (d->resourceWindows);
    //     while (i.hasNext()) {
    //         i.next();

    //         QStringList list = i.value().toList();
    //         result << QString::number((qlonglong)i.key()) + ':' + list.join(" ") + '\n';
    //     }
    // }

    return result.join("\n");
}

QString ActivityManager::_serviceIteration() const
{
    return "0.1.0";
}

#include "activitymanager.moc"
#include "activitymanager_p.moc"
