/*
 * Copyright (c) 2010 Ivan Cukic <ivan.cukic(at)kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef ACTIVITYMANAGER_H
#define ACTIVITYMANAGER_H

#include <kdedmodule.h>
#include <KComponentData>
#include <QStringList>

class KDE_EXPORT ActivityManager : public KDEDModule
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.ActivityManager")

public:
    /**
     * Creates a new ActivityManager
     */
    ActivityManager(QObject *parent, const QList<QVariant>&);

    /**
     * Destroys this ActivityManager
     */
    virtual ~ActivityManager();

public Q_SLOTS:
    /**
     * @returns the id of the current activity
     */
    QString CurrentActivity() const;

    /**
     * Sets the current activity
     * @param id id of the activity to make current
     */
    bool SetCurrentActivity(const QString & id);

    /**
     * Adds a new activity
     * @param name name of the activity
     * @returns id of the newly created activity
     */
    QString AddActivity(const QString & name);

    /**
     * Stashes the specified activity
     * @param id id of the activity to stash
     */
    void StashActivity(const QString & id);

    /**
     * Removes the specified activity
     * @param id id of the activity to delete
     */
    void RemoveActivity(const QString & id);

    /**
     * @returns the list of all existing activities
     */
    QStringList AvailableActivities() const;

    /**
     * @returns the list of all running activities
     */
    QStringList RunningActivities() const;

    /**
     * @returns the list of all stashed activities
     */
    QStringList StashedActivities() const;

    /**
     * @returns the name of the specified activity
     * @param id id of the activity
     */
    QString ActivityName(const QString & id) const;

    /**
     * Sets the name of the specified activity
     * @param id id of the activity
     * @param name name to be set
     */
    void SetActivityName(const QString & id, const QString & name);

    /**
     * @returns the description of the specified activity
     * @param id id of the activity
     */
    QString ActivityDescription(const QString & id) const;

    /**
     * Sets the description of the specified activity
     * @param id id of the activity
     * @param description description to be set
     */
    void SetActivityDescription(const QString & id, const QString & description);

    /**
     * @returns the icon of the specified activity
     * @param id id of the activity
     */
    QString ActivityIcon(const QString & id) const;

    /**
     * Sets the icon of the specified activity
     * @param id id of the activity
     * @param icon icon to be set
     */
    void SetActivityIcon(const QString & id, const QString & icon);

    // Resource related methods

    /**
     * Should be called when the client application accesses
     * the file but is not interested in registering more advanced
     * events like open/modify/close.
     * @param application unformatted name of the application
     * @param uri uri of the resource
     */
    void NotifyResourceAccessed(const QString & application, const QString & uri);

    /**
     * Should be called when the client application
     * opens a new resource identifiable by an uri.
     * @param application unformatted name of the application
     * @param windowId ID of the window that registers the resource
     * @param uri uri of the resource
     */
    void NotifyResourceOpened(const QString & application, uint windowId, const QString & uri);

    /**
     * Should be called when the client application
     * modifies a resource already registered with NotifyResourceOpened
     * @param windowId ID of the window that registers the resource
     * @param uri uri of the resource
     */
    void NotifyResourceModified(uint windowId, const QString & uri);

    /**
     * Should be called when the client application
     * closes a resource previously registered with
     * NotifyResourceOpened.
     * @param ID of the window that unregisters the resource
     * @param uri uri of the resource
     */
    void NotifyResourceClosed(uint windowId, const QString & uri);

    /**
     * @returns the list of activities that are associated with
     * the specified resource
     * @param uri uri of the resource
     */
    QStringList ActivitiesForResource(const QString & uri);

Q_SIGNALS:
    /**
     * This signal is emitted when the global
     * activity is changed
     * @param id id of the new current activity
     */
    void CurrentActivityChanged(const QString & id);

    /**
     * This signal is emitted when a new activity is created
     * @param id id of the activity
     */
    void ActivityAdded(const QString & id);

    /**
     * This signal is emitted when an activity is stashed
     * @param id id of the activity
     */
    void ActivityStashed(const QString & id);

    /**
     * This signal is emitted when an activity is deleted
     * @param id id of the activity
     */
    void ActivityRemoved(const QString & id);

    /**
     * Emitted when an activity name, description and/or icon are changed
     * @param id id of the changed activity
     */
    void ActivityChanged(const QString & id);

    /**
     * @see NotifyResourceAccessed
     */
    void ResourceAccessed(const QString & application, const QString & uri);

    /**
     * @see NotifyResourceOpened
     */
    void ResourceOpened(const QString & application, uint windowId, const QString & uri);

    /**
     * @see NotifyResourceModified
     */
    void ResourceModified(uint windowId, const QString & uri);

    /**
     * @see NotifyResourceClosed
     */
    void ResourceClosed(uint windowId, const QString & uri);

public Q_SLOTS:
    // Temporary functions
    QString _allInfo() const;
    QString _serviceIteration() const;

private:
    class Private;
    Private * const d;
};

#endif /*ACTIVITYMANAGER_H*/
