/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PROPERTYEDITWIDGET_H
#define PROPERTYEDITWIDGET_H

#include <QWidget>

#include <Nepomuk/Types/Property>
#include <Nepomuk/Variant>

#include "nepomuksearchgui_export.h"

class QLabel;
class QMouseEvent;
class QFocusEvent;

namespace Nepomuk
{

    /**
     * A widget which provides displaying and editing of one Nepomuk::Property.
     */

class NEPOMUKSEARCHGUI_EXPORT PropertyEditWidget : public QWidget
{
    Q_OBJECT

public:
    /**
     * Creates a PropertyEditWidget for the given property and value.
     * Initially the property and value are dispalyed as QLabels.
     *
     * @param parent the parent of the widget being built
     *
     * @param property the property that will be edited through this widget
     *
     * @param value the current value of the property
     */
    PropertyEditWidget(QWidget *parent = 0, const Types::Property &property = Types::Property(), const Nepomuk::Variant &value = Nepomuk::Variant());
    ~PropertyEditWidget();

    /**
     * Sets the property which will be displayed/edited through this widget to @p property.
     */
    void setNepomukProperty(const Types::Property &property);

    /**
     * Sets the value of the property to @p value.
     */
    void setValue(const Variant &value);

    /**
     * The property currently handled by this widget.
     *
     * @return the given property
     */
    Types::Property nepomukProperty() const;

    /**
     * The value of the property currently handled by this widget.
     *
     * @return the given value
     */
    Variant value() const;

Q_SIGNALS:
    void geometryChanged();

public Q_SLOTS:
    /**
     * Enables or disables editing
     */
    void setEditingEnabled(bool enable);

protected:
    /**
     * @reimp
     */
    void mouseDoubleClickEvent ( QMouseEvent * event );

    /**
     * @reimp
     */
    void focusOutEvent ( QFocusEvent * event );

    /**
      * @reimp
      * Used to catch focusOutEvents of child widgets
      */
    bool eventFilter(QObject *obj, QEvent *event);

private:
    class Private;
    Private * const d;

    /**
     * Utility function.
     * @returns value from @p editor, based on the current type
     */
    Nepomuk::Variant valueFromEditor(QWidget *editor);

    /**
     * Utility function.
     * @returns the localized label of @p resource, based on the current type
     */
    QString labelFromValue(const Nepomuk::Variant &variant);

    /**
     * Utility function.
     * @returns a new editor widget set to @p value, based on the current type and value
     */
    QWidget *createNewEditor(Nepomuk::Variant &value);

private Q_SLOTS:
    /**
     * Changed the mode to Display and displays the property and its value.
     * If an editor exists, it will be deleted.
     * @see edit()
     */
    void display();

    /**
     * Changed the mode to Edit and creates a corresponding editor for the given type.
     * @see display()
     */
    void edit();

    /**
     * Retrieves the new value from the editor(s).
     */
    void updateValue();

    /**
      * Adds an editor to a multi-valued property
      */
    void addMultipleValueEditor();

    /**
     * Copies the label of the current resource to the clipboard
     */
    void copyLabelToClipboard();

    /**
     * Copies the URI of the current resource to the clipboard
     */
    void copyValueToClipboard();

    /**
      * Constructs a string from the labels
      */
    QString labelsToString();

    /**
      * Constructs a string from the values
      */
    QString valuesToString();
};

}

#endif // PROPERTYEDITWIDGET_H
