/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourceeditwidget.h"

#include "abstractresourceeditplugin.h"
#include "resourceeditpluginfactory.h"

#include <QWidget>
#include <QVariant>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QTime>
#include <QAction>

#include <KIcon>
#include <KDebug>
#include <KPushButton>
#include <KMenu>

#include <Nepomuk/Query/QueryServiceClient>
#include <Nepomuk/Query/Result>
#include <Nepomuk/Resource>
#include <Nepomuk/KRatingWidget>

class Nepomuk::ResourceEditWidget::Private
{
public:
    Nepomuk::Resource resource;
    Nepomuk::AbstractResourceEditPlugin *editPlugin;
    QLabel *iconLabel;
    KPushButton *btnActionsMenu;
    KRatingWidget *ratingWidget;
    /*KPushButton *btnEditSave;
    KPushButton *btnCancel;
    bool editingEnabled;
    */
};

Nepomuk::ResourceEditWidget::ResourceEditWidget(QWidget *parent, const Nepomuk::Resource &resource) :
        QWidget(parent),
        d(new Private())
{
    setResource(resource);
}

Nepomuk::ResourceEditWidget::~ResourceEditWidget()
{
    delete d;
}

void Nepomuk::ResourceEditWidget::initGui()
{
    //cleanup
    if(layout()) {
        layout()->invalidate();
    }
    delete layout();
    delete d->editPlugin;
    delete d->iconLabel;
    delete d->ratingWidget;
    delete d->btnActionsMenu;


    QVBoxLayout *mainLayout = new QVBoxLayout(this);

    //setup EditPlugin
    d->editPlugin = ResourceEditPluginFactory::resourceEditPluginForType(d->resource.resourceType());
    if(d->editPlugin) {
        d->editPlugin->setResource(d->resource);
        connect(d->editPlugin, SIGNAL(geometryChanged()), this, SIGNAL(geometryChanged()));
    }

    //icon label and rating widget
    d->iconLabel = new QLabel();//this);//FIXME? parenting to the widget first results in some UI rubbish
    d->iconLabel->setPixmap(KIcon(d->resource.genericIcon()).pixmap(QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium)));

    d->ratingWidget = new KRatingWidget();//this);//FIXME? parenting to the widget first results in some UI rubbish
    d->ratingWidget->setMaxRating(10);
    d->ratingWidget->setRating(d->resource.rating());
    connect(d->ratingWidget, SIGNAL(ratingChanged(int)), this, SLOT(setRating(int)));

    //Actions, Edit and Cancel buttons
    d->btnActionsMenu = new KPushButton(i18n("Actions"));//, this);//FIXME? parenting to the widget first results in some UI rubbish
    if(d->editPlugin) {
        KMenu *actionsMenu = new KMenu(d->btnActionsMenu);
        actionsMenu->addActions(d->editPlugin->actions());
        d->btnActionsMenu->setMenu(actionsMenu);
    }

    //layout holding icon, actions menu, rating, edit, cancel
    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setContentsMargins(11, 1, 1, 1);
    hLayout->addWidget(d->iconLabel, 1);
    hLayout->addWidget(d->ratingWidget, 1);
    hLayout->addStretch();
    hLayout->addWidget(d->btnActionsMenu, 1);
    //FIXME: the actions button is painted on the left side and resides there after the button is moved to the right side of the layout

    //layout and config
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addLayout(hLayout);
    if(d->editPlugin) {
        mainLayout->addWidget(d->editPlugin);
    } else {
        QLabel *label = new QLabel(i18n("Error loading plugin"), this);
        label->setEnabled(false);
        mainLayout->addWidget(label);        
    }
    setLayout(mainLayout);

    setFocusPolicy(Qt::StrongFocus);
    //d->editingEnabled = false;
    updateGeometry();
}

Nepomuk::Resource Nepomuk::ResourceEditWidget::resource() const
{
    return d->resource;
}

void Nepomuk::ResourceEditWidget::setResource(const Resource &resource)
{
    d->resource = resource;
    initGui();
}

void Nepomuk::ResourceEditWidget::setEditingEnabled(bool enable)
{
    d->editPlugin->setEditingEnabled(enable);
    //updateGeometry();
    //d->ratingWidget->setEnabled(enable);
}

void Nepomuk::ResourceEditWidget::updateGeometry()
{
    QWidget::updateGeometry();
    emit geometryChanged();
}


void Nepomuk::ResourceEditWidget::revertChanges()
{
    d->editPlugin->revertChanges();
}


void Nepomuk::ResourceEditWidget::saveChanges()
{
    d->editPlugin->saveChanges();
}


void Nepomuk::ResourceEditWidget::setRating(int rating)
{
    d->resource.setRating(rating);
}
