/*
  Copyright (C) 2010 Michael Leupold <lemma@confuego.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef PAM_KWALLET_OPENER_H
#define PAM_KWALLET_OPENER_H

#include <QtCore/QObject>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <QtCrypto/QtCrypto>
#include <QtDBus/QDBusServiceWatcher>
#include <KDebug>
#include <KLocalizedString>

class Opener : public QObject
{
   Q_OBJECT

public:
   /**
    * Constructor
    */
   Opener();

   /**
    * Destructor
    */
   ~Opener();

   /**
    * Start trying to open the wallets.
    *
    * @remarks if kwalletd isn't running yet, the opener will wait for it to get available
    *          on the session bus and initiate opening once it's there.
    */
   void start();

   /**
    * Try opening the specified wallets.
    *
    * @return true if KWalletD was available for opening (regardless of whether opening
    *         succeeded), false if KWalletD wasn't available yet.
    */ 
   bool tryOpen();

   /**
    * Try launching the wallet daemon.
    *
    * @return true if klauncher was available for starting the daemon, false if something
    *         went wrong.
    */
   bool startDaemon();

   /**
    * Set the hashed password to use for opening.
    */
   void setHashedPassword(const QCA::SecureArray &hashedPw) {
      m_hashedPw = hashedPw;
   }
   
   /**
    * Set to true to try opening the local wallet (false by default).
    */
   void setOpenLocalWallet(bool open) {
      m_openLocalWallet = open;
   }

   /**
    * Set to false to not try opening the network wallet (true by default).
    */
   void setOpenNetworkWallet(bool open) {
      m_openNetworkWallet = open;
   }

   /**
    * Set the maximum number of miliseconds to wait for KWalletD to get available on the
    * session bus (default is 30000ms).
    */
   void setMaxwait(uint maxwait) {
      m_maxwait = maxwait;
   }

   /**
    * Set the number of miliseconds to keep the opened wallets open so other applications
    * can pick them up. After that amount of time the wallets are automatically closed
    * if no application is using them (default is 120000ms).
    */
   void setSessionTimeout(int sessionTimeout) {
      m_sessionTimeout = sessionTimeout;
   }

   /**
    * If set to true, the opener will try to start the daemon. If set to false, it is
    * assumed that the daemon is already running.
    */
   void setStartDaemon(bool startDaemon) {
      m_startDaemon = startDaemon;
   }

private Q_SLOTS:
   /**
    * Called when the org.kde.klauncher service is registered.
    */
   void slotKLauncherRegistered(const QString &serviceName);

   /**
    * Called when the org.kde.kwalletd service is registered.
    */
   void slotKWalletDRegistered(const QString &serviceName);

   /**
    * Called when the maxwait timeout is reached. Prints an error and exits the
    * application.
    */
   void timeout();

   /**
    * Used to start the service via the event-loop.
    */
   void realStart();

private:
   // two watchers are needed due to a bug up to Qt 4.6.2.
   QDBusServiceWatcher m_klauncherWatcher;
   QDBusServiceWatcher m_kwalletdWatcher;
   QTextStream m_stderr;
   
   QCA::SecureArray m_hashedPw;
   bool m_openLocalWallet;
   bool m_openNetworkWallet;
   bool m_startDaemon;
   uint m_maxwait;
   int m_sessionTimeout;
};

#endif
