/***********************************************************************************
* Mini Player: Advanced media player for Plasma.
* Copyright (C) 2008 - 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "MiniPlayerPlayListModel.h"
#include "MiniPlayerApplet.h"

#include <KIcon>
#include <KLocale>
#include <KMimeType>

#include <Phonon/MediaObject>

MiniPlayerPlayListModel::MiniPlayerPlayListModel(MiniPlayerApplet *parent) : QAbstractTableModel(parent)
{
    m_player = parent;

    setSupportedDragActions(Qt::MoveAction);

    m_currentTrack = -1;
}

void MiniPlayerPlayListModel::addTracks(KUrl::List items, int index)
{
    if (index == -1)
    {
        index = m_urls.count();
    }

    for (int i = 0; i < items.count(); ++i)
    {
        int indexOf = m_urls.indexOf(items.at(i));

        if (indexOf > -1)
        {
            if (indexOf < index)
            {
                --index;
            }

            m_urls.removeAll(items.at(i));
        }

        m_urls.insert((m_urls.begin() + index + i), items.at(i));
    }

    m_urls.removeAll(KUrl());

    reset();

    emit needsSaving();
}

void MiniPlayerPlayListModel::removeTracks(KUrl::List items)
{
    for (int i = 0; i < items.count(); ++i)
    {
        m_urls.removeAll(items.at(i));
    }

    reset();

    emit needsSaving();
}

void MiniPlayerPlayListModel::setCurrentTrack(int track)
{
    m_currentTrack = track;

    reset();
}

void MiniPlayerPlayListModel::clearPlayList()
{
    m_urls.clear();

    reset();

    emit needsSaving();
}

void MiniPlayerPlayListModel::sort(int column, Qt::SortOrder order)
{
    Q_UNUSED(column)

    if (m_urls.count() < 2)
    {
        return;
    }

    qSort(m_urls.begin(), m_urls.end());

    if (order == Qt::AscendingOrder)
    {
        KUrl::List urls;

        for (int i = (m_urls.count() - 1); i >= 0; --i)
        {
            urls.append(m_urls.at(i));
        }

        m_urls = urls;
    }

    reset();

    emit needsSaving();
}

KUrl::List MiniPlayerPlayListModel::getTracks()
{
    return m_urls;
}

QVariant MiniPlayerPlayListModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || (index.row() >= m_urls.count()))
    {
        return QVariant();
    }

    if (role == Qt::DecorationRole && index.column() == 0 && m_urls.at(index.row()).isValid())
    {
        return ((m_urls.at(index.row()) == m_player->currentUrl())?KIcon("media-playback-start"):m_player->iconForUrl(m_urls.at(index.row())));
    }
    else if (role == Qt::DisplayRole || role == Qt::EditRole)
    {
        return m_player->titleForUrl(m_urls.at(index.row()));
    }
    else if (role == Qt::ToolTipRole)
    {
        return m_urls.at(index.row()).pathOrUrl();
    }

    return QVariant();
}

QVariant MiniPlayerPlayListModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    Q_UNUSED(section)

    if (orientation != Qt::Horizontal || role != Qt::DisplayRole)
    {
        return QVariant();
    }

    return i18n("Title");
}

Qt::ItemFlags MiniPlayerPlayListModel::flags(const QModelIndex &index) const
{
    Qt::ItemFlags defaultFlags = QAbstractTableModel::flags(index);

    if (index.isValid())
    {
        return (defaultFlags | Qt::ItemIsDragEnabled | Qt::ItemIsEditable);
    }
    else
    {
        return (defaultFlags | Qt::ItemIsDropEnabled);
    }
}

Qt::DropActions MiniPlayerPlayListModel::supportedDropActions() const
{
    return (Qt::CopyAction | Qt::MoveAction);
}

QMimeData *MiniPlayerPlayListModel::mimeData(const QModelIndexList &indexes) const
{
    KUrl::List urls;

    foreach (const QModelIndex &index, indexes)
    {
        if (index.isValid() && (index.column() == 0))
        {
            urls.append(m_urls.at(index.row()));
        }
    }

    QMimeData *mimeData = new QMimeData();
    urls.populateMimeData(mimeData);

    return mimeData;
}

QStringList MiniPlayerPlayListModel::mimeTypes() const
{
    return QStringList("text/uri-list");
}

int MiniPlayerPlayListModel::columnCount(const QModelIndex &index) const
{
    if (index.isValid())
    {
        return 0;
    }

    return 1;
}

int MiniPlayerPlayListModel::rowCount(const QModelIndex &index) const
{
    if (index.isValid())
    {
        return 0;
    }

    return m_urls.count();
}

int MiniPlayerPlayListModel::currentTrack() const
{
    return m_currentTrack;
}

bool MiniPlayerPlayListModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid() || value.toString().isEmpty() || index.row() >= m_urls.count())
    {
        return false;
    }

    if (role == Qt::EditRole)
    {
        m_player->changeUrlTitle(m_urls.at(index.row()), value.toString());
    }
    else if (role == Qt::ToolTipRole)
    {
        m_urls.replace(index.row(), KUrl(value.toString()));
    }
    else
    {
        return false;
    }

    emit needsSaving();
    emit itemChanged(index);

    return true;
}

bool MiniPlayerPlayListModel::dropMimeData(const QMimeData *mimeData, Qt::DropAction action, int row, int column, const QModelIndex &index)
{
    Q_UNUSED(column)

    if (action == Qt::IgnoreAction)
    {
        return true;
    }

    if (m_urls.count() && m_urls.at(0).protocol() == "disc")
    {
        return false;
    }

    int position = m_urls.count();

    if (index.isValid())
    {
        position = index.row();
    }
    else if (row != -1)
    {
        position = row;
    }

    if (!mimeData->hasUrls())
    {
        return false;
    }

    KUrl::List urls = KUrl::List::fromMimeData(mimeData);

    if (urls.count() && urls.at(0).protocol() == "disc")
    {
        return false;
    }

    m_player->addToPlayList(urls, false, position);

    emit needsSaving();

    return true;
}

bool MiniPlayerPlayListModel::insertRows(int row, int count, const QModelIndex &index)
{
    if (!index.isValid() || row < 0 || row >= m_urls.count())
    {
        return false;
    }

    int end = (row + count);

    beginInsertRows(index, row, (end - 1));

    for (int i = 0; i < count; ++i)
    {
        m_urls.insert((m_urls.begin() + row + i), KUrl());
    }

    endInsertRows();

    if (m_currentTrack >= end)
    {
        m_currentTrack += count;
    }

    emit needsSaving();

    return true;
}

bool MiniPlayerPlayListModel::removeRows(int row, int count, const QModelIndex &index)
{
    if (!index.isValid() || row < 0 || row >= m_urls.count())
    {
        return false;
    }

    int end = (row + count);

    beginRemoveRows(index, row, (end - 1));

    m_urls.erase((m_urls.begin() + row), (m_urls.begin() + end));

    endRemoveRows();

    if (m_currentTrack >= end)
    {
        m_currentTrack -= count;
    }

    emit needsSaving();

    return true;
}

#include "MiniPlayerPlayListModel.moc"
