/*
 * Copyright (C) 2008 Laura Dragan <aprilush@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "nepomuktags_applet.h"
#include <Plasma/Theme>
#include <QGraphicsLinearLayout>
#include <QVariant>
#include <KConfigDialog>
#include <KRun>

NepomukTagsApplet::NepomukTagsApplet(QObject *parent, const QVariantList &args) :
        Plasma::Applet(parent, args)
{
    m_colorScheme = 0;
    kDebug();
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(true);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(300, 300);
}

NepomukTagsApplet::~NepomukTagsApplet()
{
   delete m_colorScheme;
}

void NepomukTagsApplet::init()
{
    kDebug();

    KConfigGroup cg = config();
    m_minSize = cg.readEntry("minSize", 12);
    m_maxSize = cg.readEntry("maxSize", 35);

    m_engine = dataEngine("nepomuktags");
    if (!m_engine->isValid()) {
        setFailedToLaunch(true, i18n("Failed to load NepomukTags DataEngine"));
        return;
    }

    m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View,
                                     Plasma::Theme::defaultTheme()->colorScheme());
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, this);
    layout->setSpacing(5);
    layout->setContentsMargins(5, 5, 5, 5);

    m_label = new Plasma::Label(this);connect(m_label->nativeWidget(), SIGNAL(linkActivated(const QString &)), this, SLOT(openSearch(const QString &)));
    layout->addItem(m_label);

    setLayout(layout);

    // m_engine->connectSource("all", this, m_refresh * 60 * 1000);
    m_engine->connectSource("all", this);
}

void NepomukTagsApplet::dataUpdated(const QString &, const Plasma::DataEngine::Data &data)
{
    if (data.isEmpty()) {
        return;
    }
    if (data["Error"].toBool()) {
        // TODO make it show some error message in the widget
        m_label->setText(QString("<html><body>ERROR!</body></html>"));
        return;
    }
    kDebug() << "Refreshing tags";
    m_tagCount = data["tags"].toMap();
    m_min = data["min"].toInt();
    m_max = data["max"].toInt();
    updateText();
}

void NepomukTagsApplet::createConfigurationInterface(KConfigDialog *parent)
{
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));

    QWidget *configWidget = new QWidget();
    m_configUi.setupUi(configWidget);

    m_configUi.minSize->setValue(m_minSize);
    m_configUi.maxSize->setValue(m_maxSize);

    parent->addPage(configWidget, i18n("General"), icon());
}

void NepomukTagsApplet::configAccepted()
{
    int minSize = m_configUi.minSize->value();
    int maxSize = m_configUi.maxSize->value();
    bool changed = false;

    KConfigGroup cg = config();

    if (m_minSize != minSize) {
        changed = true;
        m_minSize = minSize;
        cg.writeEntry("minSize", m_minSize);
    }
    if (m_maxSize != maxSize) {
        changed = true;
        m_maxSize = maxSize;
        cg.writeEntry("maxSize", m_maxSize);
    }

    if (changed) {
        updateText();
        emit configNeedsSaving();
    }
}

void NepomukTagsApplet::updateText()
{
    const QColor c = m_colorScheme->foreground().color();
    const QString rgb = QString("#%1%2%3").arg(c.red(), 0, 16).arg(c.green(), 0, 16).arg(c.blue(), 0, 16);
    QString
    html =
        QString(
            "<html><head><style type=\"text/css\">a { text-decoration:none; color: %1"
            "} a:hover {text-decoration: underline ; color:red }</style></head><body><center>").arg(
            rgb);
    QList<QString> labels = m_tagCount.keys();
    int no = m_maxSize - m_minSize;
    int dist = m_max - m_min;
    foreach(const QString &label, labels) {
        int count = m_tagCount.value(label).toInt();
        int size = m_minSize + (count * dist / no);
        //  QString tagtext = QString("<a style=\"text-decoration:none;\" href=\"nepomuksearch:/hastag:%1\"><font style=\"font-size:%2px\" color=%3>%4</font></a> ").arg(label).arg(QString::number(size)).arg(rgb).arg(label);
        const QString tagtext = QString("<a href=\"nepomuksearch:/hastag:%1\"><font style=\"font-size:%2px\">%3</font></a> ").arg(label).arg(QString::number(size)).arg(label);
        html.append(tagtext);
    }
    html.append("</center></body></html>");
    m_label->setText(html);
    kDebug() << html;
}

void NepomukTagsApplet::openSearch(const QString &link)
{
    kDebug() << "following link:" << link;
    KRun::runCommand(QString("konqueror %1").arg(link), 0);
}

void NepomukTagsApplet::themeChanged()
{
    delete m_colorScheme;
    m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View,
                                     Plasma::Theme::defaultTheme()->colorScheme());

    updateText();
}

K_EXPORT_PLASMA_APPLET(nepomuktags, NepomukTagsApplet)

#include "nepomuktags_applet.moc"
