/*
 *     Copyright (C) 2007 the ktimetracker developers
 *     Copyright (C) 2008 Laurent Montel <montel@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the
 *      Free Software Foundation, Inc.
 *      51 Franklin Street, Fifth Floor
 *      Boston, MA  02110-1301  USA.
 *
 */

#include "ktimetrackerapplet.h"


#include <KIcon>
#include <KLocale>
#include <KMessageBox>
#include <KRun>
#include <KUrl>
#include <QGraphicsLinearLayout>
#include <QDBusInterface>
#include <QDBusReply>
#include <KConfigDialog>

#include <plasma/svg.h>
#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/iconwidget.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>


KTimetrackerLaunchWidget::KTimetrackerLaunchWidget( QGraphicsWidget *parent )
    : QGraphicsProxyWidget( parent )
{
    m_layout = new QGraphicsLinearLayout(this);
    m_layout->setOrientation(Qt::Vertical);

    m_errorLabel = new Plasma::Label(this);
    m_errorLabel->setText(i18n( "Ktimetracker was not launched" ));
    m_errorLabel->nativeWidget()->setAlignment(Qt::AlignCenter);

    m_icon = new Plasma::IconWidget(KIcon("ktimetracker"),"", this);

    m_launchButton = new Plasma::PushButton(this);
    m_launchButton->setText(i18n("Launch KTimetracker"));

    m_layout->addItem(m_errorLabel);
    m_layout->addItem(m_icon);
    m_layout->addItem(m_launchButton);

    setLayout(m_layout);

    connect(m_launchButton, SIGNAL(clicked()), SLOT(startApplication()));

}

KTimetrackerLaunchWidget::~KTimetrackerLaunchWidget()
{
}

void KTimetrackerLaunchWidget::startApplication()
{
  KRun::run( "ktimetracker", KUrl::List(), 0 );
}


KTimetrackerAppWidget::KTimetrackerAppWidget( QGraphicsWidget *parent )
    : QGraphicsProxyWidget( parent )
{
    m_layout = new QGraphicsLinearLayout(this);
    m_layout->setOrientation(Qt::Vertical);

    m_icon = new Plasma::IconWidget(KIcon("ktimetracker"),"", this);
    m_icon->setMinimumSize(m_icon->sizeFromIconSize(16));

    mCurrentTask = new Plasma::Label( this );
    mCurrentTask->nativeWidget()->setAlignment(Qt::AlignCenter);

    m_start = new Plasma::IconWidget;
    m_start->setIcon("media-playback-start");
    connect(m_start, SIGNAL(clicked()), this, SLOT(startCurrentTask()));
    m_start->setMinimumSize(m_start->sizeFromIconSize(16));

    m_stop = new Plasma::IconWidget;
    m_stop->setMinimumSize(m_stop->sizeFromIconSize(16));
    m_stop->setIcon("media-playback-stop");
    connect(m_stop, SIGNAL(clicked()), this, SLOT(stopCurrentTask()));
    m_stop->setMinimumSize(m_stop->sizeFromIconSize(16));

    m_layout->addItem(m_icon);
    m_layout->addItem( mCurrentTask );

    QGraphicsLinearLayout *horizLayout = new QGraphicsLinearLayout;
    horizLayout->addItem( m_stop );
    horizLayout->addItem( m_start );

    m_layout->addItem( horizLayout );

    setLayout(m_layout);

}

KTimetrackerAppWidget::~KTimetrackerAppWidget()
{
}

void KTimetrackerAppWidget::changeActiveLabelTask( const QString &taskName )
{
    mCurrentTask->setText( taskName );
    updateButtons();
}

void KTimetrackerAppWidget::updateButtons()
{
    if ( mCurrentTask->text().isEmpty() )
    {
        m_start->setEnabled( false );
        m_stop->setEnabled( false );
    }
    else
    {
        bool activeTask = currentTaskIsActive();
        m_start->setEnabled( !activeTask );
        m_stop->setEnabled( activeTask );
    }
}

bool KTimetrackerAppWidget::currentTaskIsActive() const
{
    kDebug();
    QDBusInterface iface( "org.kde.ktimetracker", "/KTimeTracker", "org.kde.ktimetracker.ktimetracker", QDBusConnection::sessionBus() );
    QDBusReply<bool> reply = iface.call( "isTaskNameActive", mCurrentTask->text() );
    bool res = false;
    if ( !reply.isValid() )
    {
        KMessageBox::information( 0, i18n( "Current task (%1) cannot be started", mCurrentTask->text() ) );
        return res;
    }
    res = reply;
    return res;
}

void KTimetrackerAppWidget::startCurrentTask()
{
    kDebug();
    QDBusInterface iface( "org.kde.ktimetracker", "/KTimeTracker", "org.kde.ktimetracker.ktimetracker", QDBusConnection::sessionBus() );
    QDBusReply<void> reply = iface.call( "startTimerForTaskName", mCurrentTask->text() );
    if ( !reply.isValid() )
        KMessageBox::information( 0, i18n( "Current task (%1) cannot be started", mCurrentTask->text() ) );
    updateButtons();
}

void KTimetrackerAppWidget::stopCurrentTask()
{
    kDebug();
    QDBusInterface iface( "org.kde.ktimetracker", "/KTimeTracker", "org.kde.ktimetracker.ktimetracker", QDBusConnection::sessionBus() );
    QDBusReply<void> reply = iface.call( "stopTimerForTaskName", mCurrentTask->text() );
    if ( !reply.isValid() )
        KMessageBox::information( 0, i18n( "Current task (%1) cannot be stopped", mCurrentTask->text() ) );
    updateButtons();
}



KTimetrackerApplet::KTimetrackerApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args), mLaunchWidget( 0 ), mAppWidget( 0 )
{
  setHasConfigurationInterface( true );
  setBackgroundHints(NoBackground);


}

void KTimetrackerApplet::init()
{
    mLayout = new QGraphicsLinearLayout(this);
    mLayout->setContentsMargins(0, 0, 0, 0);
    mLayout->setSpacing(0);
    mLayout->setOrientation(Qt::Vertical);
    mLayout->addItem( mAppWidget );
    setLayout( mLayout );

    resize( QSize( 300, 300 ) );

    mEngine = dataEngine( "ktimetracker" );
    if ( mEngine )
    {
        mEngine->connectSource( "timetracker", this );
        mEngine->setProperty("refreshTime", 6000);
        dataUpdated( "timetracker", mEngine->query( "timetracker" ) );
    }
    else
    {
        kDebug()<<" timetracker engine can't be loaded";
    }
}

void KTimetrackerApplet::addLaunchTimerTrackerWidget()
{
   if ( !mLaunchWidget )
   {
       mLayout->removeAt( 0 );

       delete mAppWidget;
       mAppWidget=0L;

       mLaunchWidget = new KTimetrackerLaunchWidget( this );
       mLayout->addItem( mLaunchWidget );
       mLaunchWidget->resize( 300, 300 );

   }
}

void KTimetrackerApplet::addAppWidget()
{
    if ( !mAppWidget )
    {
        mLayout->removeAt( 0 );
        delete mLaunchWidget;
        mLaunchWidget=0L;

        mAppWidget=new KTimetrackerAppWidget( this );
        mLayout->addItem( mAppWidget );
        mAppWidget->resize( 300, 300 );
    }
}

void KTimetrackerApplet::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *widget = new QWidget();
    mConfigWidget.setupUi( widget );
    parent->addPage(widget, parent->windowTitle(), icon());
    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );

    connect( parent, SIGNAL( applyClicked() ),
             this, SLOT( configAccepted() ) );
    connect( parent, SIGNAL( okClicked() ), this, SLOT( configAccepted() ) );

    mConfigWidget.taskName->addItems(mEngine->query( "timetracker" )["tasks"].toStringList() );

}

void KTimetrackerApplet::configAccepted()
{
    if ( mAppWidget )
    {
        mAppWidget->changeActiveLabelTask( mConfigWidget.taskName->currentText() );
    }
}

void KTimetrackerApplet::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
{
  Q_UNUSED(source);
  bool started = data["ktimetracker-running"].toBool();
  if ( started ) {
      addAppWidget();
  } else {
      addLaunchTimerTrackerWidget();
  }
  update();
}

KTimetrackerApplet::~KTimetrackerApplet()
{
    delete mLaunchWidget;
    delete mAppWidget;
}

#include "ktimetrackerapplet.moc"
