/***************************************************************************
*   Copyright 2007 by Aaron Seigo <aseigo@kde.org>                        *
*   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
***************************************************************************/

#include "clock.h"

#include <math.h>

#include <QGraphicsScene>
#include <QPainter>
#include <QPainterPath>
#include <QPixmap>
#include <QStyleOptionGraphicsItem>
#include <QCheckBox>
#include <QPushButton>
#include <QTimeLine>
#include <QGraphicsGridLayout>
#include <QSequentialAnimationGroup>
#include <QPropertyAnimation>
#include <QAbstractTransition>
#include <QSizePolicy>
#include <QState>
#include <QStateMachine>

#include <KConfigDialog>
#include <KDebug>
#include <KLocale>
#include <KIcon>
#include <KIconLoader>
#include <KSharedConfig>
#include <KGlobalSettings>

#include <Plasma/Dialog>
#include <Plasma/FrameSvg>
#include <Plasma/PaintUtils>
#include <Plasma/Svg>
#include <Plasma/SvgWidget>
#include <Plasma/Theme>

#include "clocknumber.h"

Clock::Clock(QObject *parent, const QVariantList &args)
    : ClockApplet(parent, args),
    m_showSecondHand(false),
    m_showTimezoneString(false),
    m_showingTimezone(false),
    m_tzFrame(0),
    m_repaintCache(RepaintAll),
    m_faceCache(QPixmap()),
    m_handsCache(QPixmap()),
    m_glassCache(QPixmap()),
    m_secondHandUpdateTimer(0),
    m_animateSeconds(false)
{
    KGlobal::locale()->insertCatalog("libplasmaclock");
    // this catalog is only used once on the first start of the clock to translate the timezone in the configuration file
    KGlobal::locale()->insertCatalog("timezones4");
    setHasConfigurationInterface(true);
    resize(256, 256);
    setPreferredSize(256, 256);
    setAspectRatioMode(Plasma::Square);
    
    m_theme = new Plasma::Svg(this);
    m_theme->setImagePath("widgets/train-clock");
    m_theme->setContainsMultipleImages(true);
    m_theme->resize(size());
    kDebug() << m_theme->isValid();
    
    connect(m_theme, SIGNAL(repaintNeeded()), this, SLOT(repaintNeeded()));
}

Clock::~Clock()
{
}

void Clock::init()
{
    ClockApplet::init();
    
    KConfigGroup cg = config();
    m_showSecondHand = cg.readEntry("showSecondHand", false);
    m_showTimezoneString = cg.readEntry("showTimezoneString", false);
    m_showingTimezone = m_showTimezoneString;
//     m_fancyHands = cg.readEntry("fancyHands", false);
    setCurrentTimezone(cg.readEntry("timezone", localTimezone()));
    
//     if (m_showSecondHand) {
//         //We don't need to cache the applet if it update every seconds
//         setCacheMode(QGraphicsItem::NoCache);
//     } else {
//         setCacheMode(QGraphicsItem::DeviceCoordinateCache);
//     }

    // Set default spacings
    m_horizontalSpacing = 4;
    m_verticalSpacing = 3;
    m_margin = 10;

    connectToEngine();
    updateSizes();
    initElements();
}

void Clock::connectToEngine()
{
    m_lastTimeSeen = QTime();
    
    Plasma::DataEngine* timeEngine = dataEngine("time");
//     if (m_showSecondHand) {
        timeEngine->connectSource(currentTimezone(), this, 500);
//     } else {
//         timeEngine->connectSource(currentTimezone(), this, 6000, Plasma::AlignToMinute);
//     }
}

void Clock::constraintsEvent(Plasma::Constraints constraints)
{
    ClockApplet::constraintsEvent(constraints);
    
    if (constraints & Plasma::FormFactorConstraint) {
//         setBackgroundHints(NoBackground);
    }
    
    if (constraints & Plasma::SizeConstraint) {
        QSize pixmapSize = size().toSize();
        
        if (m_showingTimezone) {
            QRect tzArea = tzRect();
            pixmapSize.setHeight(qMax(10, pixmapSize.height() - tzArea.height()));
            tzFrame()->resizeFrame(tzArea.size());
        }
        
        pixmapSize.setWidth(pixmapSize.height());
        m_faceCache = QPixmap(pixmapSize);
        m_handsCache = QPixmap(pixmapSize);
        m_glassCache = QPixmap(pixmapSize);
        
//         updateSizes();
//         m_theme->resize(pixmapSize);
    }
    
    m_repaintCache = RepaintAll;
}

void Clock::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
{
    Q_UNUSED(source);
    m_time = data["Time"].toTime();
    m_date = data["Date"].toDate();
    
    if (m_time.minute() == m_lastTimeSeen.minute() &&
        m_time.second() == m_lastTimeSeen.second()) {
        // avoid unnecessary repaints
        return;
    }
    
    if (m_time.minute() != m_lastTimeSeen.minute()) {
        m_repaintCache = RepaintHands;
    }
    
    if (Plasma::ToolTipManager::self()->isVisible(this)) {
        updateTipContent();
    }
    
    if (m_secondHandUpdateTimer) {
        m_secondHandUpdateTimer->stop();
    }
    
    m_animateSeconds = true;
    m_lastTimeSeen = m_time;
//     if (m_time.minute() != m_lastTimeSeen.minute()) {
//         m_lastTimeSeen = m_time;
//         animateUpdate();
//     }
//     update();
//     adjustElements();
    animateUpdate();
    
    speakTime(m_time);
}

void Clock::createClockConfigurationInterface(KConfigDialog *parent)
{
    //TODO: Make the size settable
    QWidget *widget = new QWidget();
    ui.setupUi(widget);
    parent->addPage(widget, i18n("Appearance"), "view-media-visualization");
    
//     ui.showTimezoneStringCheckBox->setChecked(m_showTimezoneString);
}

void Clock::clockConfigAccepted()
{
    KConfigGroup cg = config();
//     m_showTimezoneString = ui.showTimezoneStringCheckBox->isChecked();
//     m_showingTimezone = m_showTimezoneString || shouldDisplayTimezone();
    cg.writeEntry("showTimezoneString", m_showTimezoneString);
    update();
    
    dataEngine("time")->disconnectSource(currentTimezone(), this);
    connectToEngine();
    
    constraintsEvent(Plasma::AllConstraints);
    emit configNeedsSaving();
}

void Clock::changeEngineTimezone(const QString &oldTimezone, const QString &newTimezone)
{
    dataEngine("time")->disconnectSource(oldTimezone, this);
    Plasma::DataEngine* timeEngine = dataEngine("time");
    
//     if (m_showSecondHand) {
        timeEngine->connectSource(newTimezone, this, 5000);
//     } else { FIXME reenable
//         timeEngine->connectSource(newTimezone, this, 6000, Plasma::AlignToMinute);
//     }
    
    if (m_showingTimezone != (m_showTimezoneString || shouldDisplayTimezone())) {
        m_showingTimezone = !m_showingTimezone;
        constraintsEvent(Plasma::SizeConstraint);
    }
    m_repaintCache = RepaintAll;
}

#undef waaa
#ifndef waaa
void Clock::initElements()
{
    QGraphicsGridLayout *l = new QGraphicsGridLayout;
    for (int i = 0; i < 5; i++) {
        Plasma::SvgWidget *w = new Plasma::SvgWidget(m_theme, QString(), this);
        l->addItem(w, 0, i);
        w = new Plasma::SvgWidget(m_theme, QString(), this);
        l->addItem(w, 1, i);
    }
    
    m_animatingWidget = new Plasma::SvgWidget(m_theme, QString(), this);
    m_animatingWidget->hide();
    m_animatingWidget2 = new Plasma::SvgWidget(m_theme, QString(), this);
    m_animatingWidget2->hide();
    setLayout(l);
}

void Clock::adjustElements()
{
    kDebug();
    m_animatingWidget->hide();
    m_animatingWidget2->hide();
    
    QGraphicsGridLayout *l = dynamic_cast<QGraphicsGridLayout *>(layout());
    
    if(!l) return;
    
    if (m_time.isValid()) {
        for (int i = 0; i < 5; i++) {
            
            if (i == 2) { // the :
                Plasma::SvgWidget *item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(0, i));
                item->setElementID("colon-top");
                item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(1, i));
                item->setElementID("colon-bottom");
                continue;
            }
            
            Plasma::SvgWidget *item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(0, i));
            item->setElementID("e"+timeString()[i]+"-p1");
            item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(1, i));
            item->setElementID("e"+timeString()[i]+"-p2");
        }
    }
}

void Clock::animateUpdate()
{
    QGraphicsGridLayout *l = dynamic_cast<QGraphicsGridLayout *>(layout());
    
    if(!l) return;
    Plasma::SvgWidget *item;
    
    item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(0, 4));
    QRectF start1Rect = item->geometry();
    QRectF end1Rect(QPointF(item->geometry().x(), item->geometry().y()+item->geometry().height()),
                   QSize(item->geometry().width(), 0));
                   
    item = dynamic_cast<Plasma::SvgWidget *>(l->itemAt(1, 4));
    QRectF start2Rect(QPointF(item->geometry().x(), item->geometry().y()), QSize(item->geometry().width(), 0));
    QRectF end2Rect = item->geometry();
    
//     QPropertyAnimation *flip1 = new QPropertyAnimation(m_animatingWidget, "geometry");
    
    QString time = timeString();
    int second = time[4].digitValue();
//     second++;
    
//     item->setElementID('e'+QString::number(second)+"-p1");
    
//     m_animatingWidget->setElementID('e'+QString::number(second)+"-p1");
//     kDebug() << m_animatingWidget;
//     m_animatingWidget->show();
//     QPropertyAnimation *flip2 = new QPropertyAnimation(m_animatingWidget2, "geometry");
//     flip1->setDuration(300);
//     flip1->setStartValue(item->geometry());
//     
//     flip1->setEndValue(endRect);
//     
    
//     m_animatingWidget2->setElementID(
//     m_animatingWidget2->setElementID('e'+QString::number(second+1)+"-p2");
//     m_animatingWidget2->show();
    
//     QPropertyAnimation *elementChange = new QPropertyAnimation(m_animatingWidget, "elementID");
//     elementChange->setDuration(10);
//     kDebug() << m_animatingWidget->elementID();
//     elementChange->setStartValue(m_animatingWidget->elementID());
//     elementChange->setEndValue(QString("e0-p2"));
    
//     flip2->setDuration(300);
    
//     flip2->setStartValue(startRect);
//     flip2->setEndValue(item->geometry());
//     flip2->start();    
    
    
    QPropertyAnimation *flip1 = new QPropertyAnimation(m_animatingWidget, "geometry");
    flip1->setDuration(300);
    flip1->setEasingCurve(QEasingCurve::InSine);
    QPropertyAnimation *flip2 = new QPropertyAnimation(m_animatingWidget, "geometry");
    flip2->setDuration(300);
    flip2->setEasingCurve(QEasingCurve::OutSine);
    
//     flip2->setEasingCurve(QEasingCurve::OutSine);
    QStateMachine *states = new QStateMachine(this);
    QState *rootState = new QState;
    QState *beginState = new QState(rootState);
    QState *halfFlip = new QState(rootState);
    QState *endState = new QState(rootState);
    
    beginState->assignProperty(m_animatingWidget, "visible", true);
    beginState->assignProperty(m_animatingWidget, "elementID", 'e'+QString::number(second)+"-p1");
    beginState->assignProperty(m_animatingWidget, "geometry", start1Rect);
    halfFlip->assignProperty(m_animatingWidget, "geometry", end1Rect);
    halfFlip->assignProperty(m_animatingWidget, "elementID", 'e'+QString::number(second)+"-p1");
    endState->assignProperty(m_animatingWidget, "geometry", end2Rect);
    endState->assignProperty(m_animatingWidget, "visible", false);
    
    states->addState(rootState);
    states->setInitialState(rootState);
    rootState->setInitialState(beginState);
    QAbstractTransition *trans = rootState->addTransition(halfFlip);
    trans->addAnimation(flip1);
    trans = rootState->addTransition(endState);
    trans->addAnimation(flip2);
    
    states->start();
    
//     halfFlip.assignProperty(m_animatingWidget, "geometry", endRect);
//     end.assignProperty(m_animatingWidget, "geometry", endRect);
/*    
    NumberAnimation *hide = new NumberAnimation(m_animatingWidget2, "visible");
    hide->setDuration(100);
    hide->setStartValue(true);
    hide->setEndValue(false);
    hide->start();*/
    
//     QSequentialAnimationGroup *group = new QSequentialAnimationGroup(this);
//     group->addAnimation(flip1);
    //     group->addAnimation(elementChange);
//     group->addAnimation(flip2);
//     group->addAnimation(hide);
//     group->start();
    
//         connect(flip2, SIGNAL(finished()), m_animatingWidget, SLOT(hide()));
//     connect(flip2, SIGNAL(finished()), this, SLOT(adjustElements()));
}

QString Clock::timeString()
{
    return m_time.toString("hh:ss");
}

#endif


void Clock::repaintNeeded()
{
    m_repaintCache = RepaintAll;
    update();
}

void Clock::updateSizes()
{
    m_maxHeight = qMin((int)(contentsRect().height()), m_timeRect.height());
    m_maxWidth = (int)(contentsRect().width()-(m_margin*2));
    
    if (m_maxHeight*3 > m_maxWidth) {
        m_maxHeight =(int)(m_maxWidth/3);
    } else if (m_maxHeight > m_maxWidth/3) {
        m_maxWidth = m_maxHeight*3;
    }
    
    m_timeRect.setWidth(m_maxWidth);
    m_timeRect.setHeight(m_maxHeight);
    
    while (m_timeRect.width() > m_maxWidth || m_timeRect.width() /3 > m_maxHeight) {
        m_timeRect.setHeight(m_timeRect.height()-1);
        m_timeRect.setWidth((int)(((m_timeRect.height()-m_verticalSpacing)/2) *1.4*4+(6*m_horizontalSpacing)));
    } // small enough now.
    
    
    m_timeRect = QRect(QPoint(0, 0), contentsRect().size().toSize());
    m_dateRect = QRect(0, 0, 0, 0);
    
    if (m_showDate || m_showTimezone) {
        //         kDebug() << "Show Date or Timezone";
        QFontMetrics fm(KGlobalSettings::smallestReadableFont());
        
        int smallerHeight = qRound(m_timeRect.height() - (fm.height() + m_margin*3));
        m_timeRect.setHeight(smallerHeight);   // Resize the timeRect if we have to paint some text below.
        m_dateRect.setY(smallerHeight);
        m_dateRect.setHeight((int)(contentsRect().height()));
        m_dateRect.setWidth((int)(contentsRect().width()-m_margin));
    }
    
    // enforce natural aspect ratio
    QSize elSize = m_defaultElementSize;
    int elementHeight = (int)((m_timeRect.height() - m_verticalSpacing)/2);
    int elementWidth = (int)((m_timeRect.width() - m_horizontalSpacing*4 - m_margin*2)/5);
    elSize.scale(elementWidth, elementHeight, Qt::KeepAspectRatio);
    m_elWidth = elSize.width();
    m_elHeight = elSize.height();
    // colon is 42 wide, digits 72
    m_colonWidth = (int)(m_elWidth/1.7);
    kDebug() << "el W, H, colon" << m_elWidth << m_elHeight << m_colonWidth;
    
    m_verticalSpacing = m_elHeight/11; // From the SVG,the horizontal space is 1/11 of the height
    m_horizontalSpacing = qMin(4, qRound(m_elWidth/10));
}

void Clock::finishAnimation()
{
    m_animating = false;
    m_lastTimeSeen = m_time;
    update();
    kDebug() << "finished";
}

void Clock::animationSlot(const int step)
{
    //     kDebug();
    if (step == 4) // The animation is stopped
    {
        m_animating = false;
        m_lastTimeSeen = m_time;
        update();
    } else {
        m_animating = true;
        m_animationStep = step;
        kDebug() << "step:" << m_animationStep;
        update();
    }
}

int Clock::getOffsetForDigit(const int digitNumber)
{
    int offset = 0;
    if (digitNumber == 0) {
        //return margin;
    }
    
    offset += (m_elWidth+m_horizontalSpacing) * digitNumber; // Add space taken by digit
    offset += (int)(( contentsRect().width()-(m_elWidth*4 + m_horizontalSpacing*4 + m_elWidth/1.7 - m_margin*2)) /2); // Align to center
    
    if (digitNumber > 2) { // There's a smaller gap between hours and minutes ...
        offset -= m_elWidth;
        offset += (int)(m_elWidth/1.7);
    }
    return offset;
}

void Clock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
{
    #if 0
//     kDebug() << "time" << m_time.isValid() << "date" << m_date.isValid();
//     p->drawRect(contentsRect());
    // TODO implement flipping seconds, disabled by default, and disable anyway when
    // Plasma::Applet::shouldConserveResources() is true
    Q_UNUSED(option);
    if (m_time.isValid() && m_date.isValid()) {
        
        p->setFont (KGlobalSettings::smallestReadableFont());
        p->setPen(QPen(m_fgColor));
//         p->setRenderHint(QPainter::SmoothPixmapTransform);
        p->setRenderHint(QPainter::Antialiasing);
        p->setRenderHint(QPainter::TextAntialiasing);
//         p->translate(0.5, 0.5);
        
        // Paint the text
        QString dateString;
        if (m_showDate || m_showTimezone) {
            
            if (m_showDate) {
                dateString = m_date.toString("d MMM");
                if (m_showDay) {
                    QString weekday = QDate::shortDayName(m_date.dayOfWeek());
                    dateString = weekday + ", "  + dateString;
                }
                if (m_showYear) {
                    dateString = dateString + m_date.toString(" yyyy");
                }
            }
            
            if (m_showTimezone) {
                dateString = dateString + " " + m_prettyTimezone;
                dateString.replace("_", " ");
            }
            
            p->drawText(m_dateRect, dateString,
                        QTextOption (Qt::AlignCenter));
//                         kDebug() << "Painting" << dateString;
        }
        
        m_theme->resize(m_timeRect.width(), m_timeRect.height());
        
        // Conditionally paint panel Background
        if (formFactor() == Plasma::Horizontal) {
            m_theme->paint(p, contentsRect(), "panel-bg");
        }
        
        // set left offset of clock elements so as to horizontally center the time display
        int leftOffset = getOffsetForDigit(0);
        int upperElementTop = (int)m_margin;
        //int upperElementTop = 0;
        int bottomElementTop = upperElementTop + m_elHeight + m_verticalSpacing;

        QString show_time;
//         if (m_twentyFour) { 
            show_time = m_time.toString("hh:ss");
//         } else {
//             show_time = m_time.toString("hh:mm AP");
//         }
        const QString timeString = show_time;

//         kDebug() << "time:" << timeString;
        
        // 10-hours-digit
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p2");

        
        // 1-hour-digit
        leftOffset = getOffsetForDigit(1);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p2");

        // colon
        leftOffset = getOffsetForDigit(2);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_colonWidth, m_elHeight), "colon-top");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_colonWidth, m_elHeight), "colon-bottom");

        // 10-minutes-digit
        leftOffset = getOffsetForDigit(3);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p2");

        // 1-minute-digit
        leftOffset = getOffsetForDigit(4);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p2");

        // Make sure we don't get artifacts if an update gets called while animating
        if (m_animating) {
            // If we are aninmating, this digit is for sure.
            leftOffset = getOffsetForDigit(3);
            Number oldMinutes =(QChar)timeString[4]; // 1-minutes digit
            --oldMinutes; // This is the digit which should be painted under the new one
            QString element;

            m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p1");
            element = QChar('e') +oldMinutes+QString("-p2");
            m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), element);

            if (0 < m_animationStep && m_animationStep < 3) {
                element = QChar('e') +oldMinutes+QString("-p1");
                m_theme->paint(p, QRectF(leftOffset, bottomElementTop-(m_elHeight/m_animationStep),
                                            m_elWidth, m_elHeight/m_animationStep), element);
            } else {
                m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight/2), 'e'+timeString[4]+"-p2");
            }

            if (oldMinutes == '9') {
                // Animate the 10-min digit if we have to
                Number oldMinutes = (QChar) timeString[3]; // 10-minutes digit
                --oldMinutes;
                QString element;
                
                m_theme->paint(p, QRectF(99, 20, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p1");
                element = QString('e') + oldMinutes+QString("-p2");
                m_theme->paint(p, QRectF(99, 41, m_elWidth, m_elHeight), element);
                
                if (m_animationStep < 3 && m_animationStep > 0) {
                    element = QString('e') + oldMinutes+QString("-p1");
                    m_theme->paint(p, QRectF(99, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                } else {
                    m_theme->paint(p, QRectF(99, 41, m_elWidth, m_elHeight/2), 'e'+timeString[3]+"-p2");
                }
                
                if (oldMinutes == '9') { //NOTE: it's 9 instead of 5 because of how I defined operator--
                    Number oldHours =(QChar) timeString[1];
                    --oldHours;
                    QString element;
                    
                    m_theme->paint(p, QRectF(59, 20, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p1");
                    element = QString('e') +oldHours+QString("-p2");
                    m_theme->paint(p, QRectF(59, 41, m_elWidth, m_elHeight), element);
                    
                    if (m_animationStep < 3 && m_animationStep > 0) {
                        element = QString('e') +oldHours+QString("-p1");
                        m_theme->paint(p, QRectF(59, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                    } else {
                        m_theme->paint(p, QRectF(59, 41, m_elWidth, m_elHeight/2), 'e'+timeString[1]+"-p2");
                    }
                    
                    if (oldHours == '9') {
                        Number oldHours =(QChar) timeString[0];
                        --oldHours;
                        QString element;
                        
                        m_theme->paint(p, QRectF(39, 20, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
                        element = QString('e') +oldHours+QString("-p2");
                        m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight), element);
                        
                        if (m_animationStep < 3 && m_animationStep > 0) {
                            element = QString('e') +oldHours+QString("-p1");
                            m_theme->paint(p, QRectF(39, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                        } else {
                            m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight/(6-m_animationStep)), 'e'+timeString[0]+"-p2");
                        }
                } else {
                m_theme->paint(p, QRectF(39, 20, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
                m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p2");
                }
                }
            }
        }
    }
    #endif
}

QRect Clock::tzRect()
{
    QRect rect = contentsRect().toRect();
    QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
    QFontMetrics fm(font);
    qreal left, top, right, bottom;
    tzFrame()->getMargins(left, top, right, bottom);
    int height = top + bottom + fm.height();
    return QRect(0, rect.bottom() - height, rect.width(), height);
}

Plasma::FrameSvg *Clock::tzFrame()
{
    if (!m_tzFrame) {
        m_tzFrame = new Plasma::FrameSvg(this);
        m_tzFrame->setImagePath("widgets/background");
    }
    
    return m_tzFrame;
}

#include "clock.moc"


/***************************************************************************
*   Copyright (C) 2005,2006,2007 by Siraj Razick <siraj@kdemail.net>      *
*   Copyright (C) 2007 by Riccardo Iaconelli <riccardo@kde.org>           *
*   Copyright (C) 2007 by Sebastian Kuegler <sebas@kde.org>               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
***************************************************************************/
#if 0

#include "clock.h"
#include "clocknumber.h"

#include <math.h>

#include <QtGui/QPainter>
#include <QtGui/QStyleOptionGraphicsItem>
#include <QtGui/QSpinBox>
#include <QtCore/QTimeLine>
#include <QtGui/QGraphicsView>
#include <QtGui/QGraphicsSceneMouseEvent>

#include <KDebug>
#include <KLocale>
#include <KSharedConfig>
#include <KConfigDialog>
#include <KGlobalSettings>
#include <kdatepicker.h>
#include <plasma/theme.h>
#include <plasma/dialog.h>
#include <plasma/svg.h>


Clock::Clock(QObject *parent, const QVariantList &args)
            : Plasma::Applet(parent, args),
            m_showDate(false),
            m_showYear(false),
            m_showDay(false),
            m_showTimezone(false),
            m_twentyFour(false),
            m_elWidth(0),
            m_elHeight(0),
            m_colonWidth(0),
            m_timezone(0),
            m_time(),
            m_date(),
            m_calendar(0),
            m_layout(0),
            m_lastTimeSeen(),
            m_fgColor(),
            m_animating(false),
            m_animationStep(0),
            m_horizontalSpacing(0),
            m_verticalSpacing(0)
{
    KGlobal::locale()->insertCatalog("libplasmaclock");
    setHasConfigurationInterface(true);
    Applet::resize(150, 50);
    m_fgColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
}

void Clock::init()
{
    KConfigGroup cg = config();
    m_timezone = cg.readEntry("timezone", "Local");
    m_showTimezone = cg.readEntry("showTimezone", (m_timezone != "Local"));

    m_theme = new Plasma::Svg(this);
    m_theme->setImagePath("widgets/train-clock");
    m_theme->setContainsMultipleImages(true);
    if (!m_theme->isValid()) {
        setFailedToLaunch(true, i18n("The train-clock SVG file was not found"));
        return;
    }

    m_showDate = cg.readEntry("showDate", false);
    m_showYear = cg.readEntry("showYear", false);

    m_showDay = cg.readEntry("showDay", true);
    m_twentyFour = cg.readEntry("twentyFour", true);

    // Set default spacings
    m_horizontalSpacing = 4;
    m_verticalSpacing = 3;
    m_margin = 10;

    // take the size of the top half of the '0' number
    // to calculate the aspect ratio when drawing
    m_defaultElementSize = m_theme->elementSize("e0-p1");
    if (m_defaultElementSize.isEmpty()) {
        setFailedToLaunch(true, i18n("The train-clock SVG file has no element e0-p1"));
        return;
    }
    m_contentSize = QSizeF(125, 50);
    resize(m_contentSize);
    setMinimumSize(QSizeF(27, 11));
    dataEngine("time")->connectSource(m_timezone, this, 6000, Plasma::AlignToMinute);
    updateSizes();
}

Qt::Orientations Clock::expandingDirections() const
{
    kDebug();
    return Qt::Vertical;
}

void Clock::constraintsEvent(Plasma::Constraints constraints)
{
    m_margin = qRound(contentsRect().height() / 20);
    if (constraints & Plasma::FormFactorConstraint) {
        if (formFactor() == Plasma::Vertical ||
            formFactor() == Plasma::Horizontal) {
            m_margin = 0;
            kDebug() << "Horizontal or Vertical FormFactor";
        } else {
            kDebug() << "Other FormFactor";
        }
    }
    if (constraints & Plasma::SizeConstraint) {
        int aspect = 3.1;
        if (formFactor() == Plasma::Horizontal) {
            // We have a fixed height, set some sensible width
            setMinimumWidth(contentsRect().height() * aspect);
        } else if (formFactor() == Plasma::Vertical) {
            // We have a fixed width, set some sensible height
            setMinimumHeight((int)contentsRect().width() / aspect);
        }
        updateSizes();
    }
}

void Clock::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
{
    Q_UNUSED(source);
    m_time = data["Time"].toTime();
    m_date = data["Date"].toDate();
    m_prettyTimezone = data["Timezone"].toString();

    // avoid unnecessary repaints
    if (m_time.minute() != m_lastTimeSeen.minute()) {
        m_lastTimeSeen = m_time;
        animateUpdate();
    }
}

void Clock::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if (event->buttons() == Qt::LeftButton && rect().contains(event->pos())) {
        showCalendar(event);
    } else {
        event->ignore();
    }
}

void Clock::showCalendar(QGraphicsSceneMouseEvent *event)
{
//     kDebug();
    if (m_calendar == 0) {
        m_calendar = new Plasma::Dialog();
        //m_calendar->setStyleSheet("{ border : 0px }");
        m_layout = new QVBoxLayout();
        m_layout->setSpacing(0);
        m_layout->setMargin(0);

        /*m_calendarUi*//*.setupUi(m_calendar);*/
        m_calendar->setLayout(m_layout);
        m_calendar->setWindowFlags(Qt::Popup);
        m_calendar->adjustSize();
    }

    if (m_calendar->isVisible()) {
        m_calendar->hide();
    } else {
        kDebug();
        m_calendar->move(popupPosition(m_calendar->sizeHint()));
        m_calendar->show();
    }
}

void Clock::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *widget = new QWidget();
    ui.setupUi(widget);
    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
    parent->addPage(widget, parent->windowTitle(), "chronometer");
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));

    ui.showDate->setChecked(m_showDate);
    ui.showYear->setChecked(m_showYear);
    ui.showDay->setChecked(m_showDay);
    ui.twentyFour->setChecked(m_twentyFour);
    ui.showTimezone->setChecked(m_showTimezone);
    ui.timeZones->setSelected(m_timezone, true);
    ui.timeZones->setEnabled(m_timezone != "Local");
    ui.localTimeZone->setChecked(m_timezone == "Local");

    ui.timeZones->setSelected(m_timezone, true);
}

void Clock::configAccepted()
{
    kDebug();
    KConfigGroup cg = config();
    //QGraphicsItem::update();
    QStringList tzs = ui.timeZones->selection();
    if (ui.localTimeZone->checkState() == Qt::Checked) {
        dataEngine("time")->disconnectSource(m_timezone, this);
        m_timezone = "Local";
        dataEngine("time")->connectSource(m_timezone, this, 6000, Plasma::AlignToMinute);
        cg.writeEntry("timezone", m_timezone);
    } else if (tzs.count() > 0) {
        //TODO: support multiple timezones
        QString tz = tzs.at(0);
        if (tz != m_timezone) {
            dataEngine("time")->disconnectSource(m_timezone, this);
            // We have changed the timezone, show that in the clock, but only if this
            // setting hasn't been changed.
            ui.showTimezone->setCheckState(Qt::Checked);
            m_timezone = tz;
            dataEngine("time")->connectSource(m_timezone, this, 6000, Plasma::AlignToMinute);
        }
        cg.writeEntry("timezone", m_timezone);
    } else if (m_timezone != "Local") {
        dataEngine("time")->disconnectSource(m_timezone, this);
        m_timezone = "Local";
        dataEngine("time")->connectSource(m_timezone, this, 6000, Plasma::AlignToMinute);
        cg.writeEntry("timezone", m_timezone);
    } else {
        kDebug() << "Timezone unknown: " << tzs;
    }

    m_showDate = ui.showDate->checkState() == Qt::Checked;
    cg.writeEntry("showDate", m_showDate);
    m_showYear = ui.showYear->checkState() == Qt::Checked;
    cg.writeEntry("showYear", m_showYear);
    m_showDay = ui.showDay->checkState() == Qt::Checked;
    cg.writeEntry("showDay", m_showDay);
    m_twentyFour = ui.twentyFour->checkState() == Qt::Checked;
    cg.writeEntry("twentyFour", m_twentyFour);

    if (m_showTimezone != (ui.showTimezone->checkState() == Qt::Checked)) {
        m_showTimezone = ui.showTimezone->checkState() == Qt::Checked;
        cg.writeEntry("showTimezone", m_showTimezone);
        kDebug() << "Saving show timezone: " << m_showTimezone;
    }
    updateSizes();
    update();
    cg.config()->sync();
}

Clock::~Clock()
{
    kDebug();
    delete m_calendar;
    delete m_theme;
}

void Clock::updateSizes()
{
    m_maxHeight = qMin((int)(contentsRect().height()), m_timeRect.height());
    m_maxWidth = (int)(contentsRect().width()-(m_margin*2));

    if (m_maxHeight*3 > m_maxWidth) {
        m_maxHeight =(int)(m_maxWidth/3);
    } else if (m_maxHeight > m_maxWidth/3) {
        m_maxWidth = m_maxHeight*3;
    }

    m_timeRect.setWidth(m_maxWidth);
    m_timeRect.setHeight(m_maxHeight);

    while (m_timeRect.width() > m_maxWidth || m_timeRect.width() /3 > m_maxHeight) {
        m_timeRect.setHeight(m_timeRect.height()-1);
        m_timeRect.setWidth((int)(((m_timeRect.height()-m_verticalSpacing)/2) *1.4*4+(6*m_horizontalSpacing)));
    } // small enough now.

    m_timeRect = QRect(QPoint(0, 0), contentsRect().size().toSize());
    m_dateRect = QRect(0, 0, 0, 0);

    if (m_showDate || m_showTimezone) {
//         kDebug() << "Show Date or Timezone";
        QFontMetrics fm(KGlobalSettings::smallestReadableFont());

        int smallerHeight = qRound(m_timeRect.height() - (fm.height() + m_margin*3));
        m_timeRect.setHeight(smallerHeight);   // Resize the timeRect if we have to paint some text below.
        m_dateRect.setY(smallerHeight);
        m_dateRect.setHeight((int)(contentsRect().height()));
        m_dateRect.setWidth((int)(contentsRect().width()-m_margin));
    }
    // enforce natural aspect ratio
    QSize elSize = m_defaultElementSize;
    int elementHeight = (int)((m_timeRect.height() - m_verticalSpacing)/2);
    int elementWidth = (int)((m_timeRect.width() - m_horizontalSpacing*4 - m_margin*2)/5);
    elSize.scale(elementWidth, elementHeight, Qt::KeepAspectRatio);
    m_elWidth = elSize.width();
    m_elHeight = elSize.height();
    // colon is 42 wide, digits 72
    m_colonWidth = (int)(m_elWidth/1.7);
    //kDebug() << "el W, H, colon" << m_elWidth << m_elHeight << m_colonWidth;

    m_verticalSpacing = m_elHeight/11; // From the SVG,the horizontal space is 1/11 of the height
    m_horizontalSpacing = qMin(4, qRound(m_elWidth/10));
}

void Clock::animateUpdate()
{
//     kDebug();
    QTimeLine *tl = new QTimeLine(200, this);
    tl->setFrameRange(0, 4);
    //tl->setCurveShape(QTimeLine::EaseInCurve),
    connect(tl, SIGNAL(frameChanged(int)), this, SLOT(animationSlot(int)));
    tl->start();
}

void Clock::animationSlot(const int step)
{
//     kDebug();
    if (step == 4) // The animation is stopped
    {
        m_animating = false;
        m_lastTimeSeen = m_time;
        update();
    } else {
        m_animating = true;
        m_animationStep = step;
//         kDebug() << "step:" << m_animationStep;
        update();
    }
}

int Clock::getOffsetForDigit(const int digitNumber)
{
    int offset = 0;
    if (digitNumber == 0) {
        //return margin;
    }

    offset += (m_elWidth+m_horizontalSpacing) * digitNumber; // Add space taken by digit
    offset += (int)(( contentsRect().width()-(m_elWidth*4 + m_horizontalSpacing*4 + m_elWidth/1.7 - m_margin*2)) /2); // Align to center

    if (digitNumber > 2) { // There's a smaller gap between hours and minutes ...
        offset -= m_elWidth;
        offset += (int)(m_elWidth/1.7);
    }
    return offset;
}


void Clock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    // TODO implement flipping seconds, disabled by default, and disable anyway when
    // Plasma::Applet::shouldConserveResources() is true
    Q_UNUSED(option);
    if (m_time.isValid() && m_date.isValid()) {

        p->setFont (KGlobalSettings::smallestReadableFont());
        p->setPen(QPen(m_fgColor));
        p->setRenderHint(QPainter::SmoothPixmapTransform);
        p->setRenderHint(QPainter::Antialiasing);
        p->setRenderHint(QPainter::TextAntialiasing);
        p->translate(0.5, 0.5);

        // Paint the text
        QString dateString;
        if (m_showDate || m_showTimezone) {

            if (m_showDate) {
                dateString = m_date.toString("d MMM");
                if (m_showDay) {
                    QString weekday = QDate::shortDayName(m_date.dayOfWeek());
                    dateString = weekday + ", "  + dateString;
                }
                if (m_showYear) {
                    dateString = dateString + m_date.toString(" yyyy");
                }
            }

            if (m_showTimezone) {
                dateString = dateString + " " + m_prettyTimezone;
                dateString.replace("_", " ");
            }

            p->drawText(m_dateRect, dateString,
                        QTextOption (Qt::AlignCenter));
            kDebug() << "Painting" << dateString;
        }

        // Conditionally paint panel Background
        //if (formFactor() == Plasma::Horizontal) {
            m_theme->paint(p, contentsRect, "panel-bg");
        //}

        m_theme->resize(m_timeRect.width(), m_timeRect.height());

        // set left offset of clock elements so as to horizontally center the time display
        int leftOffset = getOffsetForDigit(0);
        int upperElementTop = (int)m_margin;
        //int upperElementTop = 0;
        int bottomElementTop = upperElementTop + m_elHeight + m_verticalSpacing;

        QString show_time;
        if (m_twentyFour) {
            show_time = m_time.toString("hh:mm");
        } else {
            show_time = m_time.toString("hh:mm AP");
        }
        const QString timeString = show_time;

        // 10-hours-digit
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p2");

        // 1-hour-digit
        leftOffset = getOffsetForDigit(1);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p2");

        // colon
        leftOffset = getOffsetForDigit(2);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_colonWidth, m_elHeight), "colon-top");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_colonWidth, m_elHeight), "colon-bottom");

        // 10-minutes-digit
        leftOffset = getOffsetForDigit(3);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p2");

        // 1-minute-digit
        leftOffset = getOffsetForDigit(4);
        m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p1");
        m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p2");

        // Make sure we don't get artifacts if an update gets called while animating
        if (m_animating) {
            // If we are aninmating, this digit is for sure.
            leftOffset = getOffsetForDigit(3);
            Number oldMinutes =(QChar)timeString[4]; // 1-minutes digit
            --oldMinutes; // This is the digit which should be painted under the new one
            QString element;

            m_theme->paint(p, QRectF(leftOffset, upperElementTop, m_elWidth, m_elHeight), 'e'+timeString[4]+"-p1");
            element = QChar('e') +oldMinutes+QString("-p2");
            m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight), element);

            if (0 < m_animationStep && m_animationStep < 3) {
                    element = QChar('e') +oldMinutes+QString("-p1");
                    m_theme->paint(p, QRectF(leftOffset, bottomElementTop-(m_elHeight/m_animationStep),
                                                m_elWidth, m_elHeight/m_animationStep), element);
            } else {
                m_theme->paint(p, QRectF(leftOffset, bottomElementTop, m_elWidth, m_elHeight/2), 'e'+timeString[4]+"-p2");
            }

            if (oldMinutes == '9') {
                // Animate the 10-min digit if we have to
                Number oldMinutes = (QChar) timeString[3]; // 10-minutes digit
                --oldMinutes;
                QString element;

                m_theme->paint(p, QRectF(99, 20, m_elWidth, m_elHeight), 'e'+timeString[3]+"-p1");
                element = QString('e') + oldMinutes+QString("-p2");
                m_theme->paint(p, QRectF(99, 41, m_elWidth, m_elHeight), element);

                if (m_animationStep < 3 && m_animationStep > 0) {
                    element = QString('e') + oldMinutes+QString("-p1");
                    m_theme->paint(p, QRectF(99, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                } else {
                    m_theme->paint(p, QRectF(99, 41, m_elWidth, m_elHeight/2), 'e'+timeString[3]+"-p2");
                }

                if (oldMinutes == '9') { //NOTE: it's 9 instead of 5 because of how I defined operator--
                    Number oldHours =(QChar) timeString[1];
                    --oldHours;
                    QString element;

                    m_theme->paint(p, QRectF(59, 20, m_elWidth, m_elHeight), 'e'+timeString[1]+"-p1");
                    element = QString('e') +oldHours+QString("-p2");
                    m_theme->paint(p, QRectF(59, 41, m_elWidth, m_elHeight), element);

                    if (m_animationStep < 3 && m_animationStep > 0) {
                        element = QString('e') +oldHours+QString("-p1");
                        m_theme->paint(p, QRectF(59, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                    } else {
                        m_theme->paint(p, QRectF(59, 41, m_elWidth, m_elHeight/2), 'e'+timeString[1]+"-p2");
                    }

                    if (oldHours == '9') {
                        Number oldHours =(QChar) timeString[0];
                        --oldHours;
                        QString element;

                        m_theme->paint(p, QRectF(39, 20, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
                        element = QString('e') +oldHours+QString("-p2");
                        m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight), element);

                        if (m_animationStep < 3 && m_animationStep > 0) {
                            element = QString('e') +oldHours+QString("-p1");
                            m_theme->paint(p, QRectF(39, 40-(m_elHeight/m_animationStep), m_elWidth, m_elHeight/m_animationStep), element);
                        } else {
                            m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight/(6-m_animationStep)), 'e'+timeString[0]+"-p2");
                        }
                    } else {
                        m_theme->paint(p, QRectF(39, 20, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p1");
                        m_theme->paint(p, QRectF(39, 41, m_elWidth, m_elHeight), 'e'+timeString[0]+"-p2");
                    }
                }
            }
        }
    }
}

#include "clock.moc"
#endif
