/*
 *   Copyright 2008 Aike J Sommer <dev@aikesommer.name>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#include "screensapplet.h"
#include "screenwidget.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QGraphicsLinearLayout>

#include <Plasma/DataEngine>
#include <Plasma/Service>
#include <Plasma/PushButton>
#include <Plasma/Meter>

#include <algorithm>

#include <kephal/outputs.h>


ScreensApplet::ScreensApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_confirmLeft(0),
    m_confirmMax(0),
    m_layout(0)
{
    setBackgroundHints(DefaultBackground);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(300, 200);
}

void ScreensApplet::init()
{
    m_layout = new QGraphicsLinearLayout(Qt::Vertical);
    
    m_screensContainer = new ScreensWidget(this);
    m_layout->addItem(m_screensContainer);
    
    m_statusLayout = new QGraphicsLinearLayout(Qt::Horizontal);
    
    m_revertButton = new Plasma::PushButton(this);
    m_revertButton->hide();
    m_revertButton->setText(i18n("Revert changes"));
    m_revertButton->setStyleSheet("background: rgba( 220, 220, 220, 30% )");
    connect(m_revertButton, SIGNAL(clicked()), this, SLOT(revert()));

    m_confirmButton = new Plasma::PushButton(this);
    m_confirmButton->hide();
    m_confirmButton->setText(i18n("Confirm changes"));
    m_confirmButton->setStyleSheet("background: rgba( 220, 220, 220, 30% )");
    connect(m_confirmButton, SIGNAL(clicked()), this, SLOT(confirm()));
    
    m_confirmMeter = new Plasma::Meter(this);
    m_confirmMeter->hide();
    m_confirmMeter->setMinimum(0);
    m_confirmMeter->setMeterType(Plasma::Meter::BarMeterHorizontal);

    m_statusLayout->addItem(m_revertButton);
    m_statusLayout->addItem(m_confirmMeter);
    m_statusLayout->addItem(m_confirmButton);
    
    m_layout->addItem(m_statusLayout);
    setLayout(m_layout);
    
    Plasma::DataEngine * engine = dataEngine("screens");
    connect(engine, SIGNAL(sourceAdded(QString)), SLOT(sourceAdded(QString)));
    engine->connectAllSources(this);
}

void ScreensApplet::buildOutputs() {
    QRect totalSize(0, 0, 0, 0);
    foreach (Plasma::DataEngine::Data data, m_screenData.values()) {
        QString id = data.value("Id").toString();
        bool activated = data.value("Activated").toBool();
        
        if (! activated) {
            ScreenWidget * s = m_screens.take(id);
            delete s;
        } else {
            QRect geom = data.value("Geometry").toRect();
            totalSize = totalSize.united(geom);
        
            ScreenWidget * s;
            OutputHandler * oh;
            if (! m_screens.contains(id)) {
                s = new ScreenWidget(m_screensContainer);
                //connect(s, SIGNAL(resizeRequested(QSize)), output, SLOT(setSize(QSize)));
                m_screens.insert(id, s);
                s->setId(id);
                
                oh = new OutputHandler(this, id);
                m_outputHandlers.insert(id, oh);
                connect(s, SIGNAL(resizeRequested(QSize)), oh, SLOT(resizeRequested(QSize)));
                connect(oh, SIGNAL(resizeRequested(QString, QSize)), this, SLOT(resizeRequested(QString, QSize)));
                connect(s, SIGNAL(moveRequested(QPoint)), oh, SLOT(moveRequested(QPoint)));
                connect(oh, SIGNAL(moveRequested(QString, QPoint)), this, SLOT(moveRequested(QString, QPoint)));
            } else {
                s = m_screens[id];
                oh = m_outputHandlers[id];
            }
            s->setScreenGeom(geom);
            
            QList<QSize> sizes;
            int numSizes = data.value("NumSizes").toInt();
            for (int i = 1; i <= numSizes; ++i) {
                sizes.append(data.value("Size " + QString::number(i)).toSize());
            }
            s->setAvailableSizes(sizes);
            
            QList<QPoint> positions;
            int numPositions = data.value("NumPositions").toInt();
            for (int i = 1; i <= numPositions; ++i) {
                positions.append(data.value("Position " + QString::number(i)).toPoint());
            }
            s->setAvailablePositions(positions);
        }
    }
    
    m_totalSize = totalSize.size();
}

void ScreensApplet::resizeRequested(QString outputId, QSize size) {
    kDebug() << "resize requested" << outputId << size;
    
    Plasma::DataEngine * engine = dataEngine("screens");
    Plasma::Service * service = engine->serviceForSource("output-" + outputId);
    KConfigGroup op = service->operationDescription("size");
    op.writeEntry("width", size.width());
    op.writeEntry("height", size.height());
    /*Plasma::ServiceJob * job =*/ service->startOperationCall(op);
}

void ScreensApplet::moveRequested(QString outputId, QPoint position) {
    kDebug() << "move requested" << outputId << position;
    
    Plasma::DataEngine * engine = dataEngine("screens");
    Plasma::Service * service = engine->serviceForSource("output-" + outputId);
    KConfigGroup op = service->operationDescription("position");
    op.writeEntry("x", position.x());
    op.writeEntry("y", position.y());
    service->startOperationCall(op);
}

void ScreensApplet::confirm() {
    kDebug() << "confirm requested";
    
    Plasma::DataEngine * engine = dataEngine("screens");
    Plasma::Service * service = engine->serviceForSource("configurations");
    KConfigGroup op = service->operationDescription("confirm");
    service->startOperationCall(op);
}

void ScreensApplet::revert() {
    kDebug() << "revert requested";
    
    Plasma::DataEngine * engine = dataEngine("screens");
    Plasma::Service * service = engine->serviceForSource("configurations");
    KConfigGroup op = service->operationDescription("revert");
    service->startOperationCall(op);
}

void ScreensApplet::sourceAdded(const QString & sourceName) {
    Q_UNUSED(sourceName);
    
    Plasma::DataEngine * engine = dataEngine("screens");
    engine->connectAllSources(this);
}

void ScreensApplet::sourceRemoved(const QString & sourceName) {
    m_screenData.remove(sourceName);
    ScreenWidget * screen = m_screens.take(sourceName);
    delete screen;
    
    buildOutputs();
}

void ScreensApplet::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED(option);
    
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);
    
    QRect drawRect = contentsRect;
    drawRect.adjust(20, 20, -20, -50);
    //drawRect.adjust(0, 0, 0, -30);
    
    qreal scale = std::min(((qreal) drawRect.width()) / m_totalSize.width(), ((qreal) drawRect.height()) / m_totalSize.height());
    
    m_screensContainer->setMinimumSize(m_totalSize * scale + QSizeF(40, 40));
    
    if (m_confirmLeft > 0) {
        m_confirmMeter->setValue(m_confirmLeft);
        m_confirmMeter->setMaximum(m_confirmMax);
        m_confirmMeter->show();
        
        m_revertButton->show();
        m_confirmButton->show();
    } else {
        m_revertButton->hide();
        m_confirmButton->hide();
        m_confirmMeter->hide();
    }
    
    foreach (ScreenWidget * screen, m_screens) {
        screen->setPos(QPointF(20, 20) + screen->screenGeom().topLeft() * scale);
        screen->setScale(scale);
        screen->update();
    }
}

void ScreensApplet::dataUpdated(const QString& sourceName, const Plasma::DataEngine::Data & data) {
    kDebug() << "data updated:" << sourceName;
    
    if (sourceName.startsWith("output-")) {
        QString id = data.value("Id").toString();
        m_screenData.insert(id, data);
        
        buildOutputs();
    } else if (sourceName == "configurations") {
        m_confirmLeft = data.value("Confirmation-Timeout").toInt();
        m_confirmMax = data.value("Confirmation-Timeout-Max").toInt();
    }
    update();
}



OutputHandler::OutputHandler(QObject * parent, QString outputId)
    : QObject(parent), m_outputId(outputId)
{
}

void OutputHandler::resizeRequested(QSize size)
{
    kDebug() << "resize requested" << m_outputId << size;
    emit resizeRequested(m_outputId, size);
}

void OutputHandler::moveRequested(QPoint position)
{
    kDebug() << "move requested" << m_outputId << position;
    emit moveRequested(m_outputId, position);
}


K_EXPORT_PLASMA_APPLET(screens, ScreensApplet)
