/*
Copyright 2009 Beat Wolf <asraniel@fryx.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License or (at your option) version 3 or any later version
accepted by the membership of KDE e.V. (or its successor approved
by the membership of KDE e.V.), which shall act as a proxy
defined in Section 14 of version 3 of the license.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <engine.h>
#include <QDebug>

Engine::Engine()
{
    m_width = 0;
    m_height = 0;
    m_particles = 0;
    m_gravityPoints = 0;

    m_inited = false;
    
    m_particlesCount = 0;
    m_type = CentralStarRotatingCircle;
}

Engine::~Engine()
{
    resetEngine();
}

bool Engine::inited() const
{
    return m_inited;
}

void Engine::resetEngine()
{
    if(m_particles){
        delete[]m_particles;  
    }
    if(m_gravityPoints){
        delete[]m_gravityPoints;
    }
    m_gravityPoints = 0;
    m_particles = 0;
    m_inited = false;
}

void Engine::setSize(int x, int y)
{
    m_width = x;
    m_height = y;
    init();
}

void Engine::setParticles(int number)
{
    if(number != m_particlesCount){
        m_particlesCount = number;
        init();
    }
}

void Engine::setType(Types type)
{
    if(type != m_type){
        m_type = type;
        init();
    }
}

Types Engine::getType() const
{
    return m_type;
}

void Engine::initRandomConditions()
{
    m_gravityPointsCount = 6;
}

void Engine::initCentralStarConditions()
{
    m_gravityPointsCount = 1;
}

void Engine::init()
{
    if(!m_width || !m_height || !m_particlesCount)
        return;
    
    resetEngine();
    switch(m_type){
      case RotatingCircles:
        m_gravityPointsCount = 4;
        break;
      case Random1:
      case Random2:
        initRandomConditions();
        break;
      case ColidingGalaxies:
        m_gravityPointsCount = 2;
        break;
      case CentralStarRotatingCircle:
      case CentralStarRandom:
      case CentralStarRotating:
      case CentralStar:
        initCentralStarConditions();
        break;
    }
    
    m_particles = new struct particle[m_particlesCount];
    m_gravityPoints = new struct gravityPoint[m_gravityPointsCount];
    
    switch(m_type){
      case Random1:
        createRandomParticlesStill();
        createRandomGravityPoints();
        break;
      case Random2:
        createRestrictedGravityPoints();
        createRandomParticles();
        break;
      case CentralStar:
        createRandomParticlesStill();
        createCentralStarGravity();
        break;
      case CentralStarRandom:
        createRandomParticles();
        createCentralStarGravity();
        break;
      case CentralStarRotating:
        createCentralStarGravity();
        createRotatingParticles();
        break;
      case CentralStarRotatingCircle:
        createCentralStarGravity();
        createRotatingParticlesCircle();
        break;
      case RotatingCircles:
        createRestrictedGravityPoints();
        createRotatingParticlesCircleMultiple();
        break;
      case ColidingGalaxies:
        createRestrictedColidingGravityPoints();
        createRotatingParticlesCircleMultiple();
        break;
    }
    
    m_lastUpdate = QTime::currentTime();
    m_inited = true;
}

void Engine::createRandomParticlesStill()
{
    for(int i = 0; i < m_particlesCount; i++){
        struct particle *p = &m_particles[i];
        p->p = Eigen::Vector2d(qrand() % m_width, qrand() % m_height);
        
        p->dir = Eigen::Vector2d(0, 0);
    }
}

void Engine::createRandomParticles()
{
    for(int i = 0; i < m_particlesCount; i++){
        struct particle *p = &m_particles[i];
        p->p = Eigen::Vector2d(qrand() % m_width, qrand() % m_height);
        
        p->dir = Eigen::Vector2d(qrand() % 60 - 30, qrand() % 60 - 30 );
    }
}

void Engine::createRotatingParticles()
{
    struct gravityPoint *g = &m_gravityPoints[0];
    
    for(int i = 0; i < m_particlesCount; i++){
        struct particle *p = &m_particles[i];
        p->p = Eigen::Vector2d(qrand() % m_width, qrand() % m_height);
        
        Eigen::Vector2d dist = g->p - p->p;
        dist.normalize();
        double temp = dist[0];
        dist[0] = -dist[1];
        dist[1] = temp;
        
        p->dir = dist * 30;
    }
}

void Engine::createCircle(Eigen::Vector2d center, int radiusA, int radiusB, int start, int end){
  
    for(int i = start; i < end; i++){
        struct particle *p = &m_particles[i];
        double grad = ((qrand() % 365) /180.) * PI;
        double r = radiusA + qrand() % radiusB;
        
        int x = center[0] + r * cos(grad);
        int y = center[1] + r * sin(grad);
        
        p->p = Eigen::Vector2d(x, y);
        
        Eigen::Vector2d dist = center - p->p;
        dist.normalize();
        double temp = dist[0];
        dist[0] = -dist[1];
        dist[1] = temp;
        
        p->dir = dist * 30;
    }
}

void Engine::createRotatingParticlesCircle()
{
    struct gravityPoint *g = &m_gravityPoints[0];
    
    createCircle(g->p, 75, 200, 0, m_particlesCount);
}

void Engine::createRotatingParticlesCircleMultiple()
{
    int start = 0;
    int step = (m_particlesCount / m_gravityPointsCount) - 1;
    int end = step;
    
    for(int i = 0; i < m_gravityPointsCount; i++){
        if(i+1 == m_gravityPointsCount){
            end = m_particlesCount;
        }
        
        struct gravityPoint *g = &m_gravityPoints[i];
        createCircle(g->p, 50, 120, start, end);
        
        start = end+1;
        end += step;        
    }
}

void Engine::createRandomGravityPoints()
{
    for(int i = 0; i < m_gravityPointsCount; i++){
        struct gravityPoint *p = &m_gravityPoints[i];
        p->p = Eigen::Vector2d(qrand() % m_width, qrand() % m_height);
        p->force = 20 + qrand() % 20;
        p->limited = false;
    }
}

void Engine::createRestrictedGravityPoints()
{
    for(int i = 0; i < m_gravityPointsCount; i++){
        struct gravityPoint *p = &m_gravityPoints[i];
        p->p = Eigen::Vector2d(qrand() % m_width, qrand() % m_height);
        p->force = 9.81;
        p->radius = 300;
        p->limited = true;
    }
}

void Engine::createRestrictedColidingGravityPoints(){
  
    struct gravityPoint *p = &m_gravityPoints[0];
    p->p = Eigen::Vector2d(m_width / 2, m_height / 2 + 100);
    p->force = 9.81;
    p->radius = 300;
    p->limited = true;
    
    p = &m_gravityPoints[1];
    p->p = Eigen::Vector2d(m_width / 2, m_height / 2 -100);
    p->force = 9.81;
    p->radius = 300;
    p->limited = true;
}

void Engine::createCentralStarGravity()
{
    struct gravityPoint *p = &m_gravityPoints[0];
    p->p = Eigen::Vector2d(m_width / 2, m_height / 2);
    p->force = 9.81;
    p->limited = false;
}

struct particle* Engine::getParticle(int index) const
{
    return &m_particles[index];
}

int Engine::particles() const
{
    return m_particlesCount;
}

int Engine::gravityPoints() const
{
    return m_gravityPointsCount;
}

struct gravityPoint* Engine::getGravityPoint(int index) const
{
    return &m_gravityPoints[index];
}

void Engine::moveParticles()
{
    QTime currentTime = QTime::currentTime();
    double secs = m_lastUpdate.msecsTo(currentTime) / 1000.;
    
    for(int i = 0; i < m_particlesCount; i++){
        struct particle *p = &m_particles[i];
        
        //change dir for all gravity points
        for(int loop = 0; loop < m_gravityPointsCount; loop++){
            struct gravityPoint *g = &m_gravityPoints[loop];
            Eigen::Vector2d dist = g->p - p->p;
            
            
            if(g->limited){
              double d;
              if((d = dist.norm()) < g->radius){
                double force = g->force - (g->force/(g->radius*g->radius))*d*d;
                p->dir += (secs * force * dist.normalized());
              }
            }else{
                p->dir += (secs * g->force * dist.normalized());
            }
        }
        
        p->p += (secs * p->dir); //change coordinates
    }
    
    m_lastUpdate = QTime::currentTime();
}

