/*
 *   Copyright 2009 Marco Martin <notmart@gmail.com>
 *   Copyright 2009 Wang Hoi <zealot.hoi@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "koratscrollwidget.h"

#include <QGraphicsSceneResizeEvent>
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QApplication>
#include <QScrollBar>
#include <QWidget>
#include <QRectF>

#include <KMimeType>
#include <KDebug>

#include <plasma/widgets/iconwidget.h>

namespace Korat
{

class ScrollWidgetPrivate
{
public:
    ScrollWidgetPrivate(ScrollWidget *parent)
        : q(parent),
          widget(0),
          dragging(false)
    {
    }

    ~ScrollWidgetPrivate()
    {
    }

    void adjustScrollbars()
    {
        leftScrollIcon->hide();
        rightScrollIcon->hide();
        if (widget->size().width() > scrollingWidget->size().width()) {
            //kDebug() << widget->pos();
            QPointF widgetPos = widget->pos();
            if (widgetPos.x() < 0) {
                leftScrollIcon->setGeometry(QRectF(0,0,16,scrollingWidget->size().height()));
                leftScrollIcon->show();
            }
            if (widgetPos.x() + widget->size().width() > scrollingWidget->size().width()) {
                rightScrollIcon->setGeometry(QRectF(scrollingWidget->size().width()-16,0,16,scrollingWidget->size().height()));
                rightScrollIcon->show();
            }
        }
    }

    void horizontalScroll(int value)
    {
        if (!dragging) {
            qreal x = qBound(
                    qMin(-widget->size().width()+scrollingWidget->size().width(),(qreal)0),
                    widget->pos().x()-value,
                    (qreal)0
                    );
            widget->setPos(x, 0);
            adjustScrollbars();
        }
    }
    void leftScroll()
    {
        horizontalScroll(-60);
    }
    void rightScroll()
    {
        horizontalScroll(60);
    }

    ScrollWidget *q;
    Plasma::IconWidget *leftScrollIcon;
    Plasma::IconWidget *rightScrollIcon;
    QGraphicsWidget *scrollingWidget;
    QGraphicsWidget *widget;
    QGraphicsLinearLayout *layout;
    QString styleSheet;
    bool dragging;
};


ScrollWidget::ScrollWidget(QGraphicsWidget *parent)
    : QGraphicsWidget(parent),
      d(new ScrollWidgetPrivate(this))
{
    setContentsMargins(0,0,0,0);
//    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    d->leftScrollIcon = new Plasma::IconWidget(this);
    d->leftScrollIcon->setSvg("widgets/arrows","left-arrow");
    d->leftScrollIcon->setMinimumWidth(16);
    d->leftScrollIcon->setMaximumWidth(16);
    d->leftScrollIcon->setDrawBackground(true);
    d->leftScrollIcon->setZValue(1000);
    d->leftScrollIcon->hide();
    connect(d->leftScrollIcon,SIGNAL(clicked()),this,SLOT(leftScroll()));

    d->rightScrollIcon = new Plasma::IconWidget(this);
    d->rightScrollIcon->setSvg("widgets/arrows","right-arrow");
    d->rightScrollIcon->setMinimumWidth(16);
    d->rightScrollIcon->setMaximumWidth(16);
    d->rightScrollIcon->setDrawBackground(true);
    d->rightScrollIcon->setZValue(1000);
    d->rightScrollIcon->hide();
    connect(d->rightScrollIcon,SIGNAL(clicked()),this,SLOT(rightScroll()));

    d->layout = new QGraphicsLinearLayout(this);
    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->scrollingWidget = new QGraphicsWidget(this);
    d->scrollingWidget->setFlag(QGraphicsItem::ItemClipsToShape, true);
    d->layout->addItem(d->scrollingWidget);
}

ScrollWidget::~ScrollWidget()
{
    delete d;
}


void ScrollWidget::setWidget(QGraphicsWidget *widget)
{
    if (d->widget && d->widget != widget) {
        d->widget->removeEventFilter(this);
        delete d->widget;
    }

    d->widget = widget;
//    widget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    widget->setParentItem(d->scrollingWidget);
    widget->setPos(QPoint(0,0));
    widget->installEventFilter(this);
    d->adjustScrollbars();
}

QGraphicsWidget *ScrollWidget::widget() const
{
    return d->widget;
}

void ScrollWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
{
    if (!d->widget) {
        return;
    }

    QGraphicsWidget::resizeEvent(event);

//    d->widget->resize(d->scrollingWidget->size());
    d->horizontalScroll(0);
}

void ScrollWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    if (!d->widget) {
        return;
    }

    QPointF deltaPos = event->pos() - event->lastPos();

    d->horizontalScroll(-deltaPos.x());
    
    d->dragging = false;
    QGraphicsWidget::mouseMoveEvent(event);
}

void ScrollWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    event->accept();
}

void ScrollWidget::wheelEvent(QGraphicsSceneWheelEvent *event)
{
    if (event->delta() > 0) {
        d->leftScroll();
    } else {
        d->rightScroll();
    }
    event->accept();
//    QGraphicsWidget::wheelEvent(event);
}

bool ScrollWidget::eventFilter(QObject *watched, QEvent *event)
{
    if (!d->widget) {
        return false;
    }

    if (watched == d->widget && event->type() == QEvent::GraphicsSceneResize) {
        d->adjustScrollbars();
        QGraphicsSceneResizeEvent *ev = dynamic_cast<QGraphicsSceneResizeEvent *>(event);
        if (ev) {
            setMinimumHeight(ev->newSize().height());
            setMaximumHeight(ev->newSize().height());
        }
    }

    return false;
}

} // namespace Plasma

#include "koratscrollwidget.moc"

