#include "nodewalker.h"

#include "node.h"

#include <qstack.h>
#include <qset.h>
#include <qhash.h>
#include <qdebug.h>

using namespace Quasar;
#if 0
//
//  A----B---D--E
//      /   /
//     C----
//
//   A-C-B-D-E
//
//  A---B---D--E
//         /   /
//        C----
//
//   A-B-C-D-E
/*
//   A---  F--
//       \    \
//   B----D----E
//       /    /
//   C--------
//
//   A-B-C-D-F-E
*/
#endif

// for node
//     move to the next
//     does it have parents other than the current
//     if so then the ones after the current are next
//        push all of them on the stack
//           move to the next of the current and repeat
//           if can't go next, pop the stack and try again
struct NodeConnector {
    NodeConnector()
        : node(0),
          inKey(0),
          outKey(0),
          receiverIdx(0)
    {}
    NodeConnector(Node *n)
        : node(n)
    {
        inputKeys   = node->inputKeys();
        outputKeys  = node->outputKeys();
        outKey      = 0;
        receiverIdx = 0;
        inKey       = node->inputKeys().count() - 1;//last one
    }

    NodeConnector(Node *n, const QString &path, Node::Connection previous)
        : node(n), prev(previous)
    {
        inputKeys   = node->inputKeys();
        outputKeys  = node->outputKeys();
        outKey      = 0;
        receiverIdx = 0;

        for (int i = 0; i < inputKeys.count(); ++i) {
            if (inputKeys[i] == path) {
                inKey = i;
            }
        }
    }

    NodeConnector(const NodeConnector &other)
    {
        *this = other;
    }
    NodeConnector & operator=(const NodeConnector &o)
    {
        node        = o.node;
        inKey       = o.inKey;
        outKey      = o.outKey;
        receiverIdx = o.receiverIdx;
        inputKeys   = o.inputKeys;
        outputKeys  = o.outputKeys;
        return *this;
    }
    Node *nextNode() const
    {
        if (!hasNext())
            return 0;
        QList<Node*> nodes= node->receiversFor(outputKeys[outKey]);
        return nodes[receiverIdx];
    }
    void moveNext()
    {
        if (!hasNext())
            return;
        ++receiverIdx;
        QList<Node*> nodes= node->receiversFor(outputKeys[outKey]);
        if (receiverIdx >= nodes.count()) {
            receiverIdx = 0;
            ++outKey;
        }
    }
    bool hasNext() const
    {
        if (outputKeys.isEmpty())
            return false;
        if (outKey >= outputKeys.count())
            return false;
        QList<Node*> nodes= node->receiversFor(outputKeys[outKey]);
        if (receiverIdx >= nodes.count())
            return false;
        return true;
    }
    bool hasPrev() const
    {
        if (inputKeys.isEmpty())
            return false;
        if (inKey < 0 || inKey >= inputKeys.count())
            return false;
        Node *n = node->senderTo(inputKeys[inKey]);
        return n;
    }
    Node *prevNode() const
    {
        if (!hasPrev())
            return 0;
        return node->senderTo(inputKeys[inKey]);
    }
    void movePrev()
    {
        if (hasPrev())
            return;
        --inKey;
    }
    QString currentInKey() const
    {
        return inputKeys[inKey];
    }
    QString currentOutKey() const
    {
        return outputKeys[outKey];
    }

    Node *node;
    int inKey;
    int outKey;
    int receiverIdx;
    QList<QString> outputKeys;
    QList<QString> inputKeys;
    Node::Connection prev;
};

class TopLevelFinder
{
public:
    TopLevelFinder()
    {}

    void operator()(Node *n)
    {
        if (!n->hasInputs()) {
            topLevelNodes.insert(n);
        }
        if (!n->hasOutputs()) {
            bottomLevelNodes.insert(n);
        }
    }
    QSet<Node*> topNodes() const
    {
        return topLevelNodes;
    }
    QSet<Node*> bottomNodes() const
    {
        return bottomLevelNodes;
    }
private:
    QSet<Node*> topLevelNodes;
    QSet<Node*> bottomLevelNodes;
};

class UnordererGraphWalker
{
public:
    UnordererGraphWalker()
    {}

    template <class T>
    void forEachNodeInGraph(Node *n, T &act)
    {
        seenNodes = QSet<Node*>();
        forEachRecursive(n, act);
    }
private:
    template <class T>
    bool performAction(Node *n, T &act)
    {
        if (seenNodes.contains(n))
            return false;
        seenNodes.insert(n);

        act(n);
        return true;
    }
    template <class T>
    void forEachRecursive(Node *n, T &act)
    {
        if (!performAction(n, act))
            return;
        {
            QHash<QString, Node::Connection> inputs = n->inputNodes();
            QHash<QString, Node::Connection>::const_iterator itr;
            for (itr = inputs.constBegin(); itr != inputs.constEnd(); ++itr) {
                forEachRecursive(itr.value().node, act);
            }
        }

        {
            QMultiHash<QString, Node::Connection> outputs = n->outputNodes();
            QMultiHash<QString, Node::Connection>::const_iterator itr;
            for (itr = outputs.constBegin(); itr != outputs.constEnd(); ++itr) {
                forEachRecursive(itr.value().node, act);
            }
        }
    }

private:
    QSet<Node*> seenNodes;
};

class NodeWalker::Private : public QSharedData
{
public:
    void init(Node *n)
    {
        node = n;
        UnordererGraphWalker walker;
        TopLevelFinder finder;
        walker.forEachNodeInGraph(n, finder);
        topLevelNodes = finder.topNodes();
        bottomLevelNodes = finder.bottomNodes();
    }

    Node *nextNode() const
    {
        return 0;
    }
    Node *prevNode() const
    {
        return 0;
    }

    Node *node;
    QSet<Node*> topLevelNodes;
    QSet<Node*> bottomLevelNodes;
};

NodeWalker::NodeWalker(Node *node)
    : d(new Private)
{
    d->init(node);
}

NodeWalker::~NodeWalker()
{
}

void NodeWalker::next()
{
}

void NodeWalker::prev()
{
}

bool NodeWalker::hasNext() const
{
    return false;
}

bool NodeWalker::hasPrev() const
{
    return false;
}

void NodeWalker::moveToFirstParent()
{
}

void NodeWalker::moveToChild(const QString &path)
{
    Q_UNUSED(path);
}

void NodeWalker::moveToParent(const QString &path)
{
    Q_UNUSED(path);
}

QStringList NodeWalker::childPaths() const
{
    return QStringList();
}

QStringList NodeWalker::parentPaths() const
{
    return QStringList();
}

Node * NodeWalker::currentNode() const
{
    return 0;
}

Node * NodeWalker::nextNode() const
{
    return 0;
}

Node * NodeWalker::prevNode() const
{
    return 0;
}

Node * NodeWalker::firstParent() const
{
    return 0;
}

Node * NodeWalker::firstChild() const
{
    return 0;
}

QList<Node*> NodeWalker::topLevelNodes() const
{
    if (d->topLevelNodes.isEmpty())
        return QList<Node*>();
    return d->topLevelNodes.toList();
}

NodeWalker::NodeWalker(const NodeWalker &other)
    : d(other.d)
{
    qDebug()<<"creating d " <<d;
}

NodeWalker & NodeWalker::operator=(const NodeWalker &other)
{
    d = other.d;
    return *this;
}
